"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCli = void 0;
// Ensure TypeScript is exposed before any files import it
require("../mutations/createExposedTypeScript").requireExposedTypeScript();
const chalk_1 = __importDefault(require("chalk"));
const commander_1 = require("commander");
const fs = __importStar(require("mz/fs"));
const path = __importStar(require("path"));
const index_1 = require("../index");
const initialization_1 = require("../initialization");
const output_1 = require("../output");
/**
 * Parses raw string arguments and, if they're valid, calls to a main method.
 *
 * @param rawArgv   Raw string arguments and any system dependency overrides.
 * @param mainRunner   Method to run with parsed arguments: generally `typeStat`.
 * @returns Promise for the result of running TypeStat.
 */
const runCli = (rawArgv, runtime) => __awaiter(void 0, void 0, void 0, function* () {
    const command = new commander_1.Command()
        .storeOptionsAsProperties(true)
        .option("-c --config [config]", "path to a TypeStat config file")
        .option("-l --logfile [logfile]", "path to a logfile to print detailed logs in")
        .option("-V --version", "output the package version");
    const rawOptions = command.parse(rawArgv);
    runtime !== null && runtime !== void 0 ? runtime : (runtime = {
        initializationRunner: initialization_1.initialization,
        output: output_1.createProcessOutput(rawOptions.logfile),
        mainRunner: index_1.typeStat,
    });
    if (rawArgv.length === 2) {
        const initialized = yield runtime.initializationRunner(runtime.output);
        if (initialized.status !== index_1.ResultStatus.Succeeded || !initialized.skipped) {
            return initialized.status;
        }
        rawOptions.config = "typestat.json";
    }
    if ({}.hasOwnProperty.call(rawOptions, "version")) {
        runtime.output.stdout(yield getPackageVersion());
        return index_1.ResultStatus.Succeeded;
    }
    let result;
    try {
        result = yield runtime.mainRunner(rawOptions, runtime.output);
    }
    catch (error) {
        result = {
            error: error instanceof Error ? error : new Error(error),
            status: index_1.ResultStatus.Failed,
        };
    }
    switch (result.status) {
        case index_1.ResultStatus.ConfigurationError:
            runtime.output.stdout(command.helpInformation());
            runtime.output.stderr(chalk_1.default.yellow(result.error));
            break;
        case index_1.ResultStatus.Failed:
            runtime.output.stderr(chalk_1.default.yellow(result.error));
            break;
        case index_1.ResultStatus.Succeeded:
            runtime.output.stdout(chalk_1.default.blue("All done! ✨"));
            break;
    }
    return result.status;
});
exports.runCli = runCli;
const getPackageVersion = () => __awaiter(void 0, void 0, void 0, function* () {
    const packagePath = path.join(__dirname, "../../package.json");
    const rawText = (yield fs.readFile(packagePath)).toString();
    return JSON.parse(rawText).version;
});
//# sourceMappingURL=runCli.js.map