"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
const package_json_1 = require("../../package.json");
const runCli_1 = require("./runCli");
const createTestArgs = (...argv) => ({
    argv: ["node.exe", "typestat", ...argv],
    initializationRunner: jest.fn().mockResolvedValueOnce({
        status: __1.ResultStatus.ConfigurationError,
    }),
    output: {
        log: jest.fn(),
        stderr: jest.fn(),
        stdout: jest.fn(),
    },
    mainRunner: jest.fn(),
});
describe("runCli", () => {
    it("runs initializationRunner when no args are provided", () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const { argv, initializationRunner, output, mainRunner } = createTestArgs();
        // Act
        yield runCli_1.runCli(argv, { initializationRunner, output, mainRunner });
        // Assert
        expect(initializationRunner).toHaveBeenCalledTimes(1);
        expect(mainRunner).not.toHaveBeenCalled();
    }));
    it("logs the current version when --version is provided", () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const { argv, initializationRunner, output, mainRunner } = createTestArgs("--version");
        // Act
        const resultStatus = yield runCli_1.runCli(argv, { initializationRunner, output, mainRunner });
        // Assert
        expect(output.stdout).toHaveBeenLastCalledWith(`${package_json_1.version}`);
        expect(resultStatus).toEqual(__1.ResultStatus.Succeeded);
    }));
    it("logs an error when the main runner rejects with one", () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const { argv, initializationRunner, output, mainRunner } = createTestArgs("--config", "typestat.json");
        const message = "Error message";
        mainRunner.mockRejectedValue(new Error(message));
        // Act
        const resultStatus = yield runCli_1.runCli(argv, { initializationRunner, output, mainRunner });
        // Assert
        expect(output.stderr).toHaveBeenLastCalledWith(expect.stringMatching(message));
        expect(resultStatus).toEqual(__1.ResultStatus.Failed);
    }));
    it("logs help and the error when a configuration error is reported", () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const { argv, initializationRunner, output, mainRunner } = createTestArgs("--config", "typestat.json");
        const message = "Error message";
        mainRunner.mockResolvedValue({
            error: message,
            status: __1.ResultStatus.ConfigurationError,
        });
        // Act
        const resultStatus = yield runCli_1.runCli(argv, { initializationRunner, output, mainRunner });
        // Assert
        expect(output.stdout).toHaveBeenLastCalledWith(expect.stringMatching("typestat \\[options\\]"));
        expect(output.stderr).toHaveBeenLastCalledWith(expect.stringMatching(message));
        expect(resultStatus).toEqual(__1.ResultStatus.ConfigurationError);
    }));
    it("logs a happy message when it finishes succesfully", () => __awaiter(void 0, void 0, void 0, function* () {
        // Arrange
        const { argv, initializationRunner, output, mainRunner } = createTestArgs("--config", "typestat.json");
        mainRunner.mockResolvedValue({
            status: __1.ResultStatus.Succeeded,
        });
        // Act
        const resultStatus = yield runCli_1.runCli(argv, { initializationRunner, output, mainRunner });
        // Assert
        expect(output.stdout).toHaveBeenLastCalledWith(expect.stringMatching("All done!"));
        expect(resultStatus).toEqual(__1.ResultStatus.Succeeded);
    }));
});
//# sourceMappingURL=runCli.test.js.map