"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.typeStat = exports.ResultStatus = void 0;
// Ensure TypeScript is exposed before any files import it
require("./mutations/createExposedTypeScript").requireExposedTypeScript();
const automutate_1 = require("automutate");
const loadOptions_1 = require("./options/loadOptions");
const processFileRenames_1 = require("./options/processFileRenames");
const createTypeStatProvider_1 = require("./runtime/createTypeStatProvider");
var ResultStatus;
(function (ResultStatus) {
    ResultStatus[ResultStatus["Succeeded"] = 0] = "Succeeded";
    ResultStatus[ResultStatus["Failed"] = 1] = "Failed";
    ResultStatus[ResultStatus["ConfigurationError"] = 2] = "ConfigurationError";
})(ResultStatus = exports.ResultStatus || (exports.ResultStatus = {}));
const typeStat = (argv, output) => __awaiter(void 0, void 0, void 0, function* () {
    const allOptions = yield tryLoadingOptions(argv, output);
    if (allOptions instanceof Error || typeof allOptions === "string") {
        return {
            error: allOptions,
            status: ResultStatus.ConfigurationError,
        };
    }
    for (let i = 0; i < allOptions.length; i += 1) {
        // First run any any file renames needed as part of project setup
        const options = yield processFileRenames_1.processFileRenames(allOptions[i]);
        if (typeof options === "string") {
            return {
                error: new Error(`Could not run options at index ${i}: ${options}`),
                status: ResultStatus.Failed,
            };
        }
        // Next run the actual core TypeStat mutation logic
        try {
            yield automutate_1.runMutations({
                mutationsProvider: createTypeStatProvider_1.createTypeStatProvider(options),
            });
        }
        catch (error) {
            return {
                error: error,
                status: ResultStatus.Failed,
            };
        }
    }
    return {
        status: ResultStatus.Succeeded,
    };
});
exports.typeStat = typeStat;
const tryLoadingOptions = (argv, output) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return yield loadOptions_1.loadOptions(argv, output);
    }
    catch (error) {
        return error instanceof Error ? error : new Error(error);
    }
});
//# sourceMappingURL=index.js.map