"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initialization = void 0;
const chalk_1 = __importDefault(require("chalk"));
const os_1 = require("os");
const __1 = require("..");
const errors_1 = require("../shared/errors");
const initializeJavaScript_1 = require("./initializeJavaScript");
const initializeProject_1 = require("./initializeProject");
const initializePurpose_1 = require("./initializePurpose");
const initializeTypeScript_1 = require("./initializeTypeScript");
const fileName = "typestat.json";
const initialization = (output) => __awaiter(void 0, void 0, void 0, function* () {
    output.stdout([chalk_1.default.greenBright("👋"), chalk_1.default.green(" Welcome to TypeStat! "), chalk_1.default.greenBright("👋"), chalk_1.default.reset("")].join(""));
    output.stdout([chalk_1.default.reset(`This will create a new `), chalk_1.default.yellowBright(fileName), chalk_1.default.reset(` for you.`)].join(""));
    output.stdout(`If you don't know how to answer, that's ok - just select the default answer.`);
    output.stdout(chalk_1.default.reset(""));
    let skipped;
    try {
        skipped = yield runPrompts();
    }
    catch (error) {
        output.stderr(errors_1.getQuickErrorSummary(error));
        return {
            status: __1.ResultStatus.ConfigurationError,
        };
    }
    if (!skipped) {
        output.stdout(chalk_1.default.reset(`${os_1.EOL}Awesome! You're now ready to:`));
        output.stdout(chalk_1.default.greenBright(`typestat --config ${fileName}`));
        output.stdout(chalk_1.default.reset(`${os_1.EOL}Once you run that, TypeStat will start auto-fixing your typings.`));
        output.stdout([chalk_1.default.yellow(`Please report any bugs on https://github.com/JoshuaKGoldberg/TypeStat! `), chalk_1.default.yellowBright("💖")].join(""));
    }
    output.stdout(chalk_1.default.reset(""));
    return {
        skipped,
        status: __1.ResultStatus.Succeeded,
    };
});
exports.initialization = initialization;
const runPrompts = () => __awaiter(void 0, void 0, void 0, function* () {
    const purpose = yield initializePurpose_1.initializePurpose();
    if (purpose === initializePurpose_1.InitializationPurpose.Skipped) {
        return true;
    }
    const project = yield initializeProject_1.initializeProject();
    yield (purpose === initializePurpose_1.InitializationPurpose.ConvertJavaScript ? initializeJavaScript_1.initializeJavaScript : initializeTypeScript_1.initializeTypeScript)({ fileName, project });
    return false;
});
//# sourceMappingURL=index.js.map