"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.joinAssignedTypesByName = void 0;
const types_1 = require("../shared/types");
/**
 * Joins a set of assigned type values into a single mapping by name.
 */
const joinAssignedTypesByName = (request, assignedTypeValues) => {
    const assignedTypesByName = new Map();
    for (const { name, type } of assignedTypeValues) {
        // If the type comes with its own name, it's for a single property
        if (name !== undefined) {
            assignedTypesByName.set(name, type);
            continue;
        }
        // Types without names are spread to convey multiple properties
        if (typeof type !== "string") {
            for (const property of type.getProperties()) {
                const declarations = property.getDeclarations();
                const relevantDeclaration = declarations === undefined ? property.valueDeclaration : declarations[0];
                if (relevantDeclaration === undefined) {
                    continue;
                }
                const propertyType = types_1.getTypeAtLocationIfNotError(request, relevantDeclaration);
                if (propertyType !== undefined) {
                    assignedTypesByName.set(property.name, propertyType);
                }
            }
        }
    }
    return assignedTypesByName;
};
exports.joinAssignedTypesByName = joinAssignedTypesByName;
//# sourceMappingURL=assignments.js.map