"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNoImplicitAnyMutations = exports.canNodeBeFixedForNoImplicitAny = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const types_1 = require("../../shared/types");
const creation_1 = require("./creation");
/**
 * Error codes for the TypeScript language service to get --noImplicitAny code fixes.
 */
var NoImplicitAnyErrorCode;
(function (NoImplicitAnyErrorCode) {
    NoImplicitAnyErrorCode[NoImplicitAnyErrorCode["Parameter"] = 7006] = "Parameter";
    NoImplicitAnyErrorCode[NoImplicitAnyErrorCode["PropertyOrVariable"] = 7005] = "PropertyOrVariable";
})(NoImplicitAnyErrorCode || (NoImplicitAnyErrorCode = {}));
const canNodeBeFixedForNoImplicitAny = (node) => node.type === undefined &&
    node.initializer === undefined &&
    // TypeScript still provides --noImplicitAny fixes for variables that can't receive them
    // @see https://github.com/JoshuaKGoldberg/TypeStat/issues/77
    !ts.isCatchClause(node.parent) &&
    // TypeScript provides all parameters' --noImplicitAny fixes when asked for any parameter, so only request on the first
    (!ts.isParameter(node) || node === node.parent.parameters[0]);
exports.canNodeBeFixedForNoImplicitAny = canNodeBeFixedForNoImplicitAny;
const getNoImplicitAnyMutations = (node, request) => {
    // If the node is a parameter, make sure it doesn't already have an inferable type
    // (TypeScript will still suggest a codefix to make a redundant inferred type)
    if (ts.isParameter(node)) {
        const nodeType = types_1.getTypeAtLocationIfNotError(request, node);
        if (nodeType === undefined || !tsutils.isTypeFlagSet(nodeType, ts.TypeFlags.Any)) {
            return undefined;
        }
    }
    // Retrieve code fix suggestions for --noImplicitAny from the requesting language service
    const codeFixes = getNoImplicitAnyCodeFixes(node, request);
    if (codeFixes.length === 0) {
        return undefined;
    }
    // Convert those code fix suggestions to our own mutations format
    return creation_1.createCodeFixCreationMutation(request, codeFixes, {
        ignoreKnownBlankTypes: true,
    });
};
exports.getNoImplicitAnyMutations = getNoImplicitAnyMutations;
/**
 * Uses a requesting language service to get --noImplicitAny code fixes for a type of node.
 */
const getNoImplicitAnyCodeFixes = (node, request) => request.services.languageService.getCodeFixesAtPosition(request.sourceFile.fileName, node.getStart(request.sourceFile), node.end, [ts.isParameter(node) ? NoImplicitAnyErrorCode.Parameter : NoImplicitAnyErrorCode.PropertyOrVariable], {
    insertSpaceBeforeAndAfterBinaryOperators: true,
}, {});
//# sourceMappingURL=noImplicitAny.js.map