"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNoImplicitThisMutations = void 0;
const creation_1 = require("./creation");
/**
 * Error code for the TypeScript language service to get --noImplicitThis code fixes.
 */
const noImplicitThisErrorCode = 2683;
const getNoImplicitThisMutations = (node, request) => {
    // Create a mutation for the code fixes if anything is available
    const codeFixes = getNoImplicitThisCodeFixes(node, request);
    return codeFixes.length === 0 ? undefined : creation_1.createCodeFixCreationMutation(request, codeFixes);
};
exports.getNoImplicitThisMutations = getNoImplicitThisMutations;
/**
 * Uses a requesting language service to get --noImplicitThis code fixes for a type of node.
 *
 * @param node   Requesting node to retrieve fixes on.
 * @param request   Source file, metadata, and settings to collect mutations in the file.
 */
const getNoImplicitThisCodeFixes = (node, request) => request.services.languageService.getCodeFixesAtPosition(request.sourceFile.fileName, node.getStart(request.sourceFile), node.end, [noImplicitThisErrorCode], {}, {});
//# sourceMappingURL=noImplicitThis.js.map