"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.recursivelyCollectSubTypes = exports.collectFlagsAndTypesFromTypes = exports.collectUsageFlagsAndSymbols = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const nodeTypes_1 = require("../shared/nodeTypes");
const sets_1 = require("../shared/sets");
const aliases_1 = require("./aliasing/aliases");
const flags_1 = require("./collecting/flags");
/**
 * Collects assigned and missing flags and types, recursively accounting for type unions.
 *
 * @param request   Metadata and settings to collect mutations in a file.
 * @param declaredType   Original type declared on a node.
 * @param allAssignedTypes   All types immediately or later assigned to the node.
 */
const collectUsageFlagsAndSymbols = (request, declaredType, allAssignedTypes) => {
    // Collect which flags are later assigned to the type
    const [assignedFlags, assignedTypes] = exports.collectFlagsAndTypesFromTypes(request, allAssignedTypes);
    // If the declared type is the general 'any', then we assume all are missing
    // Similarly, if it's a plain Function or Object, we'll want to replace its contents
    if (declaredType.flags & ts.TypeFlags.Any || nodeTypes_1.isKnownGlobalBaseType(declaredType)) {
        return {
            assignedFlags,
            assignedTypes,
            missingFlags: assignedFlags,
            missingTypes: assignedTypes,
        };
    }
    // Otherwise, collect which flags and types are declared (as a type annotation)...
    const [declaredFlags, declaredTypes] = exports.collectFlagsAndTypesFromTypes(request, [declaredType]);
    // Subtract the above to find any flags or types assigned but not declared
    return {
        assignedFlags,
        assignedTypes,
        missingFlags: flags_1.findMissingFlags(declaredType, assignedFlags, declaredFlags),
        missingTypes: findMissingTypes(request, assignedTypes, declaredTypes),
    };
};
exports.collectUsageFlagsAndSymbols = collectUsageFlagsAndSymbols;
/**
 * Separates raw type node(s) into their contained flags and types.
 *
 * @param options   Source file, metadata, and settings to collect mutations in the file.
 * @param allTypes   Any number of raw type nodes.
 * @param allowStrictNullCheckAliases   Whether to allow `null` and `undefined` aliases regardless of compiler strictness.
 * @returns Flags and types found within the raw type nodes.
 */
const collectFlagsAndTypesFromTypes = (request, allTypes, allowStrictNullCheckAliases) => {
    const foundFlags = new Set();
    const foundTypes = new Set();
    const applicableTypeAliases = aliases_1.getApplicableTypeAliases(request, allowStrictNullCheckAliases);
    // Scan each type for undeclared type additions
    for (const type of allTypes) {
        // For any simple type flag we later will care about for aliasing, add it if it's in the type
        for (const [typeFlag] of applicableTypeAliases) {
            if (flags_1.isTypeFlagSetRecursively(type, typeFlag)) {
                foundFlags.add(typeFlag);
            }
        }
        // If the type is a rich type (has a symbol), add it in directly
        if (type.getSymbol() !== undefined) {
            foundTypes.add(type);
            continue;
        }
        // If the type is a union, add any flags or types found within it
        if (tsutils.isUnionType(type)) {
            const subTypes = exports.recursivelyCollectSubTypes(type);
            const [subFlags, deepSubTypes] = exports.collectFlagsAndTypesFromTypes(request, subTypes);
            for (const subFlag of subFlags) {
                foundFlags.add(subFlag);
            }
            for (const deepSubType of deepSubTypes) {
                foundTypes.add(deepSubType);
            }
        }
    }
    return [foundFlags, foundTypes];
};
exports.collectFlagsAndTypesFromTypes = collectFlagsAndTypesFromTypes;
const recursivelyCollectSubTypes = (type) => {
    const subTypes = [];
    for (const subType of type.types) {
        if (tsutils.isUnionType(subType)) {
            subTypes.push(...exports.recursivelyCollectSubTypes(subType));
        }
        else {
            subTypes.push(subType);
        }
    }
    return subTypes;
};
exports.recursivelyCollectSubTypes = recursivelyCollectSubTypes;
const findMissingTypes = (request, assignedTypes, declaredTypes) => {
    // If anything is of type `any`, then bail out immediately: we have no idea what's missing
    for (const type of [...assignedTypes, ...declaredTypes]) {
        if (flags_1.isTypeFlagSetRecursively(type, ts.TypeFlags.Any)) {
            return new Set();
        }
    }
    const declaredTypesContainFunction = Array.from(declaredTypes).some(typeContainsFunction);
    const remainingMissingTypes = new Set(assignedTypes);
    const isAssignedTypeMissingFromDeclared = (assignedType) => {
        // We ignore assigned function types when the declared type(s) include function(s).
        // These non-assigned function types are more likely what users would consider bugs.
        // For example, covariant functions might not be assignable, but should be fixed manually.
        if (declaredTypesContainFunction && typeContainsFunction(assignedType)) {
            return false;
        }
        for (const potentialParentType of declaredTypes) {
            if (request.services.program.getTypeChecker().isTypeAssignableTo(assignedType, potentialParentType)) {
                return false;
            }
        }
        return true;
    };
    for (const assignedType of assignedTypes) {
        if (!isAssignedTypeMissingFromDeclared(assignedType)) {
            remainingMissingTypes.delete(assignedType);
        }
    }
    return sets_1.setSubtract(remainingMissingTypes, declaredTypes);
};
const typeContainsFunction = (type) => type.getCallSignatures().length !== 0;
//# sourceMappingURL=collecting.js.map