"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTypeFlagSetRecursively = exports.findMissingFlags = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const sets_1 = require("../../shared/sets");
const knownTypeFlagEquivalents = new Map([
    [ts.TypeFlags.BigInt, ts.TypeFlags.BigIntLiteral],
    [ts.TypeFlags.BigIntLiteral, ts.TypeFlags.BigInt],
    [ts.TypeFlags.Number, ts.TypeFlags.NumberLiteral],
    [ts.TypeFlags.NumberLiteral, ts.TypeFlags.Number],
    [ts.TypeFlags.String, ts.TypeFlags.StringLiteral],
    [ts.TypeFlags.StringLiteral, ts.TypeFlags.String],
    [ts.TypeFlags.Undefined, ts.TypeFlags.Void],
    [ts.TypeFlags.Void, ts.TypeFlags.Undefined],
]);
const findMissingFlags = (declaredType, assignedFlags, declaredFlags) => {
    // If the type is declared to allow `any`, it can't be missing anything
    if (exports.isTypeFlagSetRecursively(declaredType, ts.TypeFlags.Any)) {
        return new Set();
    }
    // Otherwise, it's all the flags assigned to it that weren't already declared
    const missingFlags = sets_1.setSubtract(assignedFlags, declaredFlags);
    // Remove any flags that are just equivalents of the existing ones
    // For example, initial presense of `void` makes `undefined` unnecessary, and vice versa
    for (const [original, equivalent] of knownTypeFlagEquivalents) {
        if (missingFlags.has(equivalent) && exports.isTypeFlagSetRecursively(declaredType, original)) {
            missingFlags.delete(equivalent);
        }
    }
    return missingFlags;
};
exports.findMissingFlags = findMissingFlags;
/**
 * Checks if a type contains a type flag, accounting for deep nested type unions.
 *
 * @param parentType   Parent type to check for the type flag.
 * @param typeFlag   Type flag to check within the parent type.
 * @returns Whether the parent type contains the type flag.
 */
const isTypeFlagSetRecursively = (parentType, typeFlag) => {
    if (tsutils.isTypeFlagSet(parentType, typeFlag)) {
        return true;
    }
    if (tsutils.isUnionOrIntersectionType(parentType)) {
        for (const childType of parentType.types) {
            if (exports.isTypeFlagSetRecursively(childType, typeFlag)) {
                return true;
            }
        }
    }
    return false;
};
exports.isTypeFlagSetRecursively = isTypeFlagSetRecursively;
//# sourceMappingURL=flags.js.map