"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addIncompleteTypesToType = void 0;
const automutate_1 = require("automutate");
const nodeTypes_1 = require("../../shared/nodeTypes");
/**
 * Appends new types as union type nodes to existing, apparently incomplete types.
 */
const addIncompleteTypesToType = (request, incompleteTypes) => {
    const mutations = [];
    for (const summaryWithNode of incompleteTypes.values()) {
        const mutation = fillInIncompleteType(request, summaryWithNode);
        if (mutation !== undefined) {
            mutations.push(mutation);
        }
    }
    return mutations.length === 0 ? undefined : automutate_1.combineMutations(...mutations);
};
exports.addIncompleteTypesToType = addIncompleteTypesToType;
const fillInIncompleteType = (request, summaryWithNode) => {
    var _a;
    // Create a new type name to add on that joins the types to be added
    let createdTypeName = request.services.printers.type(summaryWithNode.summary.types, (_a = summaryWithNode.originalProperty.type) !== null && _a !== void 0 ? _a : summaryWithNode.originalProperty);
    // For some reason, the enclosingNode option of printing isn't always applying...
    if (createdTypeName.includes("=>")) {
        createdTypeName = `(${createdTypeName})`;
    }
    // Similar to createTypeAdditionMutation, if the node is a basic base type, we can just replace it
    if (summaryWithNode.originalProperty.type !== undefined &&
        (nodeTypes_1.isKnownGlobalBaseType(summaryWithNode.originalPropertyType) || nodeTypes_1.isNeverAndOrUnknownType(summaryWithNode.originalPropertyType))) {
        return {
            insertion: `: ${createdTypeName}`,
            range: {
                begin: summaryWithNode.originalProperty.name.end,
                end: summaryWithNode.originalProperty.type.end,
            },
            type: "text-swap",
        };
    }
    // Otherwise, add to the original node property's types
    return {
        insertion: ` | ${createdTypeName}`,
        range: {
            begin: summaryWithNode.originalProperty.type.end,
        },
        type: "text-insert",
    };
};
//# sourceMappingURL=addIncompleteTypesToType.js.map