"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addMissingTypesToType = void 0;
const nodePrinting_1 = require("../../shared/printing/nodePrinting");
/**
 * Adds new named type properties to a declaration that is missing them.
 */
const addMissingTypesToType = (request, node, missingTypes) => {
    let insertion = "";
    for (const [name, summary] of missingTypes) {
        insertion += nodePrinting_1.printNamedTypeSummary(request, node, name, summary);
    }
    if (!insertion) {
        return undefined;
    }
    return {
        insertion,
        range: {
            begin: getEndInsertionPoint(node),
        },
        type: "text-insert",
    };
};
exports.addMissingTypesToType = addMissingTypesToType;
const getEndInsertionPoint = ({ end, members }) => {
    if (members.length === 0) {
        return end - 1;
    }
    const lastMember = members[members.length - 1];
    return Math.min(lastMember.end + 1, end);
};
//# sourceMappingURL=addMissingTypesToType.js.map