"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTypeExpansionMutation = void 0;
const automutate_1 = require("automutate");
const ts = __importStar(require("typescript"));
const arrays_1 = require("../../shared/arrays");
const names_1 = require("../../shared/names");
const addIncompleteTypesToType_1 = require("./addIncompleteTypesToType");
const addMissingTypesToType_1 = require("./addMissingTypesToType");
const eliminations_1 = require("./eliminations");
const summarization_1 = require("./summarization");
const nodeTypes_1 = require("../../shared/nodeTypes");
const types_1 = require("../../shared/types");
/**
 * Given an interface or type declaration and a set of later-assigned types,
 * expands the original declaration to now also include the types.
 */
const createTypeExpansionMutation = (request, node, allAssignedTypes) => {
    const originalPropertiesByName = groupPropertyDeclarationsByName(node);
    const summarizedAssignedTypes = summarization_1.summarizeAllAssignedTypes(request, allAssignedTypes);
    const incompleteTypes = new Map();
    const missingTypes = new Map();
    for (const [name, summary] of summarizedAssignedTypes) {
        // If the original type doesn't have the name at all, we'll need to add it in
        const originalProperty = originalPropertiesByName.get(name);
        if (originalProperty === undefined) {
            missingTypes.set(name, summary);
            continue;
        }
        // If the type matches an existing property in name but not in type, we'll add the new type in there
        const originalPropertyType = types_1.getTypeAtLocationIfNotError(request, originalProperty);
        if (originalPropertyType !== undefined && eliminations_1.originalTypeHasIncompleteType(request, originalPropertyType, summary.types)) {
            incompleteTypes.set(name, { originalProperty, originalPropertyType, summary });
        }
    }
    const incompleteTypesMutations = addIncompleteTypesToType_1.addIncompleteTypesToType(request, incompleteTypes);
    const missingTypesMutations = addMissingTypesToType_1.addMissingTypesToType(request, node, missingTypes);
    const mutations = [incompleteTypesMutations, missingTypesMutations].filter(arrays_1.isNotUndefined);
    return mutations.length === 0 ? undefined : automutate_1.combineMutations(...mutations);
};
exports.createTypeExpansionMutation = createTypeExpansionMutation;
const groupPropertyDeclarationsByName = (node) => {
    const propertiesByName = new Map();
    for (const member of node.members) {
        // Ignore non-existent or implicitly typed members
        if (!ts.isPropertySignature(member) || !nodeTypes_1.isNodeWithType(member)) {
            continue;
        }
        // Ignore any property with a name that's not immediately convertable to a string
        const name = names_1.getStaticNameOfProperty(member.name);
        if (name === undefined) {
            continue;
        }
        propertiesByName.set(name, member);
    }
    return propertiesByName;
};
//# sourceMappingURL=expansionMutations.js.map