# `importExtensions`

Whether to add extensions to `export` and `import` declarations that refer to file paths without them.

## Use Cases

* Your existing code uses a build tool to import from files without using extensions, but TypeScript's `declare module` syntax requires importing via extensions.

## Configuration

```json
{
    "fixes": {
        "importExtensions": true
    }
}
```

## Mutations

### Direct File Paths

If a declaration refers to a file without its extension but TypeScript would need that extension added for the import to work, it'll be added in:

```diff
- import image from './assets/image';
+ import image from './assets/image.svg';
```

### Index File Paths

Files named `index` that are imported by their directory name will be accounted for as well:

```diff
- import styles from './assets/styles';
+ import styles from './assets/styles/index.module.scss';
```
