"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectTypeParameterReferences = void 0;
const ts = __importStar(require("typescript"));
const arrays_1 = require("../../../../shared/arrays");
const nodes_1 = require("../../../../shared/nodes");
/**
 * Finds all the nodes that could indicate the type of a base type parameter,
 * by finding all references to the class's declaration nodes of that type parameter.
 */
const collectTypeParameterReferences = (request, childClass, baseClass, baseTypeParameter) => {
    // Find nodes that reference (and therefore indicate type information for) the base type
    const referencingNodes = request.fileInfoCache.getNodeReferencesAsNodes(baseTypeParameter);
    if (referencingNodes === undefined) {
        return undefined;
    }
    const expandedReferences = [];
    // For each node, find *its* references to see what types it may be
    for (const referencingNode of referencingNodes) {
        const { parent } = referencingNode;
        const expandedParentReferences = findParentExpandedReferences(request, parent);
        if (expandedParentReferences !== undefined) {
            expandedReferences.push(...expandedParentReferences);
        }
    }
    return expandedReferences;
};
exports.collectTypeParameterReferences = collectTypeParameterReferences;
const findParentExpandedReferences = (request, node) => {
    node = getEquivalentContainingTypeNode(node);
    // Property and variable declarations can be directly searched for as references
    if (ts.isParameterPropertyDeclaration(node, node.parent) || ts.isPropertyDeclaration(node) || ts.isVariableDeclaration(node)) {
        return request.fileInfoCache.getNodeReferencesAsNodes(node);
    }
    // Parameters need to check the equivalent index of their function's calls
    if (ts.isParameter(node) && ts.isFunctionLike(node.parent)) {
        const calls = request.fileInfoCache.getNodeReferencesAsNodes(node.parent);
        if (calls === undefined) {
            return undefined;
        }
        const parentIndex = node.parent.parameters.indexOf(node);
        return calls
            .map(nodes_1.getExpressionWithin)
            .map(nodes_1.getCloseAncestorCallOrNewExpression)
            .filter(arrays_1.isNotUndefined)
            .map((call) => { var _a; return (_a = call.arguments) === null || _a === void 0 ? void 0 : _a[parentIndex]; })
            .filter(arrays_1.isNotUndefined);
    }
    return undefined;
};
const getEquivalentContainingTypeNode = (node) => {
    while (ts.isIntersectionTypeNode(node.parent)) {
        node = node.parent.parent;
    }
    return node;
};
//# sourceMappingURL=collectTypeParameterReferences.js.map