"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectGenericUses = void 0;
const ts = __importStar(require("typescript"));
const types_1 = require("../../../../../shared/types");
/**
 * @returns Map of the names of a class' generic template types to the types passed to them.
 */
const collectGenericUses = (request, node, genericClassDetails) => {
    const references = request.fileInfoCache.getNodeReferencesAsNodes(node);
    if (references === undefined) {
        return undefined;
    }
    const assignedParameterTypes = new Map();
    const addAssignmentToTypeParameter = (typeParameterName, argumentType) => {
        const existing = assignedParameterTypes.get(typeParameterName);
        if (existing === undefined) {
            assignedParameterTypes.set(typeParameterName, [argumentType]);
        }
        else {
            existing.push(argumentType);
        }
    };
    // Each reference might contain a call expression to a member of the generic container
    for (const reference of references) {
        if (!ts.isExpressionStatement(reference)) {
            continue;
        }
        const callExpression = reference.expression;
        if (!ts.isCallExpression(callExpression)) {
            continue;
        }
        const callArguments = callExpression.arguments;
        if (callArguments.length === 0) {
            continue;
        }
        const propertyAccessExpression = callExpression.expression;
        if (!ts.isPropertyAccessExpression(propertyAccessExpression)) {
            continue;
        }
        // Only look at members that are known to have the generic parameters, like `indexOf(item: T)`
        const memberName = propertyAccessExpression.name.text;
        const memberDetails = genericClassDetails.membersWithGenericParameters.get(memberName);
        if (memberDetails === undefined) {
            continue;
        }
        for (const [parameterIndex, { parameterName, parameterType }] of memberDetails) {
            if (parameterIndex >= callArguments.length) {
                continue;
            }
            const parameterIndexType = types_1.getTypeAtLocationIfNotError(request, callArguments[parameterIndex]);
            if (parameterIndexType === undefined) {
                continue;
            }
            // For each parameter passed to the generic use, we'll record its assignment type
            addAssignmentToTypeParameter(parameterName, parameterIndexType);
            // If the parameter is a rest parameter, also add assignment types for any following arguments
            if (parameterType.parent.dotDotDotToken !== undefined) {
                for (const callArgument of callArguments.slice(parameterIndex + 1)) {
                    const callArgumentType = types_1.getTypeAtLocationIfNotError(request, callArgument);
                    if (callArgumentType !== undefined) {
                        addAssignmentToTypeParameter(parameterName, callArgumentType);
                    }
                }
            }
        }
    }
    return assignedParameterTypes;
};
exports.collectGenericUses = collectGenericUses;
//# sourceMappingURL=collectGenericUses.js.map