"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createExplicitGenericType = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const generics_1 = require("../../../../../mutations/generics");
const typeNodes_1 = require("../../../../../shared/typeNodes");
/**
 * @returns Test insertion mutation if a generic type should be made explicit.
 */
const createExplicitGenericType = (request, node, genericClassDetails, allAssignedGenericTypes) => {
    const allTypeArgumentTypes = [];
    // For each type parameter, collect the types assigned to it
    for (const typeParameterName of genericClassDetails.typeParameterNames) {
        const typeArgumentTypes = allAssignedGenericTypes.get(typeParameterName);
        if (typeArgumentTypes === undefined) {
            return undefined;
        }
        allTypeArgumentTypes.push(typeArgumentTypes);
    }
    // If we couldn't find any types, there's nothing to do here
    if (allTypeArgumentTypes.length === 0) {
        return undefined;
    }
    // If the node's inferred generic type already contains the later types, don't change anything
    if (allTypeArgumentTypesMatch(request, allTypeArgumentTypes, node)) {
        return undefined;
    }
    // Convert the container and its type assignments into a labeled type
    const joinedGenericType = generics_1.joinIntoGenericType(request, genericClassDetails.containerType, allTypeArgumentTypes);
    if (joinedGenericType === undefined) {
        return undefined;
    }
    return {
        insertion: `: ${joinedGenericType}`,
        range: {
            begin: node.name.end,
        },
        type: "text-insert",
    };
};
exports.createExplicitGenericType = createExplicitGenericType;
const allTypeArgumentTypesMatch = (request, allTypeArgumentTypes, node) => {
    const typeChecker = request.services.program.getTypeChecker();
    // If the original type doesn't have type arguments, bail out immediately
    const originalType = typeChecker.getTypeAtLocation(node);
    if (!typeNodes_1.isTypeArgumentsType(originalType)) {
        return false;
    }
    // If the implicit type has defaulted to any, ignore it (assume a non-match)
    if (originalType.typeArguments.some((typeArgument) => tsutils.isTypeFlagSet(typeArgument, ts.TypeFlags.Any))) {
        return false;
    }
    // If the implicit types are all unknown, assume a non-match for being unknown
    if (originalType.typeArguments.every((typeArgument) => tsutils.isTypeFlagSet(typeArgument, ts.TypeFlags.Unknown))) {
        return false;
    }
    // Otherwise, check that all type arguments sub-types match the original type at the same index
    for (const typeArgumentTypes of allTypeArgumentTypes) {
        if (typeArgumentTypes.length !== originalType.typeArguments.length) {
            return false;
        }
        for (let i = 0; i < typeArgumentTypes.length; i += 1) {
            if (!typeChecker.isTypeAssignableTo(typeArgumentTypes[i], originalType.typeArguments[i])) {
                return false;
            }
        }
    }
    return true;
};
//# sourceMappingURL=createExplicitGenericType.js.map