"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getGenericClassDetails = void 0;
const ts = __importStar(require("typescript"));
const types_1 = require("../../../../../shared/types");
const getGenericClassDetails = (request, node) => {
    // Get the backing type of the variable's initializer
    const initializerType = types_1.getTypeAtLocationIfNotError(request, node.initializer);
    const initializerSymbol = initializerType === null || initializerType === void 0 ? void 0 : initializerType.getSymbol();
    if (initializerSymbol === undefined) {
        return undefined;
    }
    // We'll be looking at the list of members available on that type
    const initializerMembers = initializerSymbol.members;
    if (initializerMembers === undefined || initializerMembers.size === 0) {
        return undefined;
    }
    // Only care about types that have at least one (local?) type parameter
    const typeChecker = request.services.program.getTypeChecker();
    const containerType = typeChecker.getDeclaredTypeOfSymbol(initializerSymbol);
    if (!types_1.typeHasLocalTypeParameters(containerType) ||
        containerType.localTypeParameters === undefined ||
        containerType.localTypeParameters.length === 0) {
        return undefined;
    }
    // Create the summary of type parameter names with relevant member functions
    return fillMembersWithGenericParameters(containerType, containerType.localTypeParameters.map((typeParameter) => typeParameter.symbol.name), initializerMembers);
};
exports.getGenericClassDetails = getGenericClassDetails;
/**
 * @returns For the container type, type parameter names with member functions that use them
 */
const fillMembersWithGenericParameters = (containerType, typeParameterNames, initializerMembers) => {
    const membersWithGenericParameters = new Map();
    const typeParameterNamesSet = new Set(typeParameterNames);
    /**
     * If a parameter's type name is a reference to a local parameter, adds it to the mapping.
     */
    const setMemberWithGenericParameterIfMatched = (memberName, parameterIndex, typeName, parameterType) => {
        if (!ts.isIdentifier(typeName)) {
            return;
        }
        const parameterName = typeName.text;
        if (!typeParameterNamesSet.has(parameterName)) {
            return;
        }
        const existing = membersWithGenericParameters.get(memberName);
        const summary = { parameterName, parameterType };
        if (existing === undefined) {
            membersWithGenericParameters.set(memberName, new Map([[parameterIndex, summary]]));
        }
        else {
            existing.set(parameterIndex, summary);
        }
    };
    // We'll be looking through all members declared on the backing initializer's type
    initializerMembers.forEach((memberSymbol) => {
        // Skip the member if it's the type itself, as that might be included in .members
        const memberName = memberSymbol.name;
        if (typeParameterNamesSet.has(memberName)) {
            return;
        }
        // We only care about parameters that can receive instances of the generic type
        const { valueDeclaration } = memberSymbol;
        if (!ts.isFunctionLike(valueDeclaration)) {
            return;
        }
        for (let i = 0; i < valueDeclaration.parameters.length; i += 1) {
            const parameter = valueDeclaration.parameters[i];
            const parameterType = parameter.type;
            if (parameterType === undefined) {
                continue;
            }
            // Case: indexOf(item: T)
            // Add the parameter if the direct T type matches
            if (ts.isTypeReferenceNode(parameterType)) {
                setMemberWithGenericParameterIfMatched(memberName, i, parameterType.typeName, parameterType);
                continue;
            }
            // Case: push(...items: T[])
            // Add the parameter if the T within the array type matches
            if (ts.isArrayTypeNode(parameterType) && ts.isTypeReferenceNode(parameterType.elementType)) {
                setMemberWithGenericParameterIfMatched(memberName, i, parameterType.elementType.typeName, parameterType);
                continue;
            }
            // There could probably be more cases here, such as `concat(...items: ConcatArray<T>[])`,
            // For now the `item: T` and `...items: T[]` cases capture most uses of built-in generics
        }
    });
    return { containerType, membersWithGenericParameters, typeParameterNames };
};
//# sourceMappingURL=getGenericClassDetails.js.map