"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixIncompleteImplicitVariableGenerics = void 0;
const collectMutationsFromNodes_1 = require("../../../../collectMutationsFromNodes");
const collectGenericUses_1 = require("./collectGenericUses");
const createExplicitGenericType_1 = require("./createExplicitGenericType");
const getGenericClassDetails_1 = require("./getGenericClassDetails");
const implicitGenericTypes_1 = require("./implicitGenericTypes");
const fixIncompleteImplicitVariableGenerics = (request) => collectMutationsFromNodes_1.collectMutationsFromNodes(request, implicitGenericTypes_1.isVariableWithImplicitGeneric, visitVariableWithImplicitGeneric);
exports.fixIncompleteImplicitVariableGenerics = fixIncompleteImplicitVariableGenerics;
const visitVariableWithImplicitGeneric = (node, request) => {
    // Get the type of class the variable is an instance of, such as [] (Array) or Map
    // If the variable's class didn't have generics, we can ignore this
    const genericClassDetails = getGenericClassDetails_1.getGenericClassDetails(request, node);
    if (genericClassDetails === undefined) {
        return undefined;
    }
    // Find places where the node was either assigned a known type or its method was called with a type
    // If the node is never used with a (non-any) type, there's nothing we can (or would want) to do
    const assignedGenericTypes = collectGenericUses_1.collectGenericUses(request, node, genericClassDetails);
    if (assignedGenericTypes === undefined || assignedGenericTypes.size === 0) {
        return undefined;
    }
    // Combine those type uses into a new explicit generic type on the node
    return createExplicitGenericType_1.createExplicitGenericType(request, node, genericClassDetails, assignedGenericTypes);
};
//# sourceMappingURL=index.js.map