"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fillInMissingTemplateTypes = void 0;
const assignments_1 = require("../../../../mutations/assignments");
const creationMutations_1 = require("../../../../mutations/creations/creationMutations");
const summarization_1 = require("../../../../mutations/expansions/summarization");
const naming_1 = require("../../../../mutations/naming");
const types_1 = require("../../../../shared/types");
const fillInMissingTemplateTypes = (request, childClass, baseTypeParameters, templateTypeLists) => {
    const createdTypes = [];
    const templateTypeNames = [];
    const childClassName = naming_1.getPerceivedNameOfClass(request, childClass);
    // Template types will be marked on the node up through the last missing type
    for (let i = 0; i < templateTypeLists.length; i += 1) {
        const templateTypes = templateTypeLists[i];
        // If the template type is fine on the node but missing, use the default from the generic
        // Example: a React component's props are {} but its state needs a new type
        if (templateTypes === undefined) {
            const defaultTemplateValue = findDefaultTemplateValue(request, baseTypeParameters[i]);
            if (defaultTemplateValue !== undefined) {
                templateTypeNames.push(defaultTemplateValue);
            }
            continue;
        }
        // If the template type is already a known name, directly use it
        if (typeof templateTypes === "string") {
            templateTypeNames.push(templateTypes);
            continue;
        }
        // In this case, the template type is missing: create a new type to fill the missing template type
        // That new type will need a name, which we'll generate from the class and relevant type parameter
        const newTemplateTypeName = createTemplateTypeName(childClassName, baseTypeParameters, baseTypeParameters[i]);
        // Properties on that template type are collected from uses of the type in the node
        const assignedTypeValues = Array.from(templateTypes).map(([name, type]) => ({ name, type }));
        const allAssignedTypes = assignments_1.joinAssignedTypesByName(request, assignedTypeValues);
        const newType = creationMutations_1.createDeclarationForTypeSummaries(request, undefined /* enclosingDeclaration */, newTemplateTypeName, summarization_1.summarizeAllAssignedTypes(request, [allAssignedTypes]));
        createdTypes.push(newType);
        templateTypeNames.push(newTemplateTypeName);
    }
    return { createdTypes, templateTypeNames };
};
exports.fillInMissingTemplateTypes = fillInMissingTemplateTypes;
const findDefaultTemplateValue = (request, baseTypeParameter) => {
    if (baseTypeParameter.default === undefined) {
        return "{}";
    }
    const baseTypeDefault = types_1.getTypeAtLocationIfNotError(request, baseTypeParameter.default);
    if (baseTypeDefault === undefined) {
        return undefined;
    }
    return request.services.printers.type(baseTypeDefault);
};
const createTemplateTypeName = (childClassName, baseTypeParameters, baseTypeParameter) => {
    const friendlyName = naming_1.getFriendlyTypeParameterDeclarationName(baseTypeParameters, baseTypeParameter);
    return childClassName + friendlyName[0].toUpperCase() + friendlyName.slice(1);
};
//# sourceMappingURL=templateMutating.js.map