"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.expandValuesAssignedToReferenceNodes = void 0;
const ts = __importStar(require("typescript"));
const nodes_1 = require("../../../../shared/nodes");
const types_1 = require("../../../../shared/types");
const expandValuesAssignedToReferenceNodes = (request, originalType, genericReferenceNodes) => {
    const assignedValues = new Set();
    for (const genericReferenceNode of genericReferenceNodes) {
        const assignedGenericValue = getAssignedGenericValue(request, originalType, genericReferenceNode);
        if (assignedGenericValue !== undefined) {
            assignedValues.add(assignedGenericValue);
        }
    }
    return Array.from(assignedValues);
};
exports.expandValuesAssignedToReferenceNodes = expandValuesAssignedToReferenceNodes;
const getAssignedGenericValue = (request, originalType, node) => {
    // Case:
    //   function container<T>(values: T) { }
    //   type Values = {};
    //   container<Values>({ laterAssigned: true, });
    // Case:
    //   declare class Container<T> { values: t; }
    //   type Values = {};
    //   new Container<Values>({ laterAssigned: true, });
    // Case:
    //   declare class Container<T> { setValues(values: t): void; }
    //   type Values = {};
    //   new Container<Values>({}).setValues({ laterAssigned: true });
    if (ts.isObjectLiteralExpression(node)) {
        return assignedTypeOrUndefined({
            type: types_1.getTypeAtLocationIfNotError(request, node),
        });
    }
    if (ts.isIdentifier(node) && ts.isPropertyAccessExpression(node.parent)) {
        // Case:
        //   declare class Container<T> { values: t; }
        //   type Values = {};
        //   const container = new Container<Values>({});
        //   container.values = { laterAssigned: true, };
        if (nodes_1.isNodeAssigningBinaryExpression(node.parent.parent) && types_1.getTypeAtLocationIfNotError(request, node.parent) === originalType) {
            return assignedTypeOrUndefined({
                type: types_1.getTypeAtLocationIfNotError(request, node.parent.parent.right),
            });
        }
        // Case:
        //   declare class Container<T> { values: T; }
        //   type Values = {};
        //   const container = new Container<Values>({});
        //   container.values.laterAssigned = true;
        // Case:
        //   declare class Container<T> { values: T; }
        //   type Values = {};
        //   new Container<Values>({}).values.quicklyAssigned = true;
        if (ts.isPropertyAccessExpression(node.parent.parent) &&
            ts.isIdentifier(node.parent.parent.name) &&
            nodes_1.isNodeAssigningBinaryExpression(node.parent.parent.parent) &&
            types_1.getTypeAtLocationIfNotError(request, node.parent) === originalType) {
            return assignedTypeOrUndefined({
                name: node.parent.parent.name.getText(),
                type: types_1.getTypeAtLocationIfNotError(request, node.parent.parent.parent.right),
            });
        }
    }
    return undefined;
};
const assignedTypeOrUndefined = (assignedType) => assignedType.type === undefined ? undefined : assignedType;
//# sourceMappingURL=expandValuesAssignedToReferenceNodes.js.map