"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixIncompleteParameterTypes = void 0;
const ts = __importStar(require("typescript"));
const creators_1 = require("../../../../mutations/creators");
const nodeTypes_1 = require("../../../../shared/nodeTypes");
const types_1 = require("../../../../shared/types");
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const fixIncompleteParameterTypes = (request) => collectMutationsFromNodes_1.collectMutationsFromNodes(request, isParameterWithType, visitParameterDeclaration);
exports.fixIncompleteParameterTypes = fixIncompleteParameterTypes;
const isParameterWithType = (node) => ts.isParameter(node) && nodeTypes_1.isNodeWithType(node);
const visitParameterDeclaration = (node, request) => {
    // Collect types initially assigned or later called with as the parameter
    const callingTypes = getCallingTypesFromReferencedSymbols(node, request);
    // Collect the type(s) initially declared on the parameter
    const declaredType = types_1.getTypeAtLocationIfNotError(request, node);
    if (declaredType === undefined) {
        return undefined;
    }
    // If the parameter already has a declared type, add assigned types to it if necessary
    if (nodeTypes_1.isNodeWithType(node)) {
        return creators_1.createTypeAdditionMutation(request, node, declaredType, callingTypes);
    }
    // Since the parameter doesn't have its own type, give it one if necessary
    return creators_1.createTypeCreationMutation(request, node, declaredType, callingTypes);
};
const getCallingTypesFromReferencedSymbols = (node, request) => {
    const callingTypes = [];
    // If the parameter has a default, also consider that a calling type
    const initializerType = types_1.getTypeAtLocationIfNotError(request, node.initializer);
    if (initializerType !== undefined) {
        callingTypes.push(initializerType);
    }
    // Find all locations the containing method is called
    const references = request.fileInfoCache.getNodeReferencesAsNodes(node.parent);
    if (references !== undefined) {
        const parameterIndex = node.parent.parameters.indexOf(node);
        // For each calling location, add any types it's called with there
        for (const reference of references) {
            updateCallingTypesForReference(parameterIndex, reference, callingTypes, request);
        }
    }
    return callingTypes;
};
const updateCallingTypesForReference = (parameterIndex, callingNode, callingTypes, request) => {
    // In order to be calling with this parameter, the referencing node should be an expression...
    if (!ts.isExpressionStatement(callingNode)) {
        return;
    }
    // ...that calls to the parameter we're looking at
    if (!ts.isCallExpression(callingNode.expression) || callingNode.expression.arguments.length <= parameterIndex) {
        return;
    }
    // Mark the type of parameter at our index as being called with
    const callingType = types_1.getTypeAtLocationIfNotError(request, callingNode.expression.arguments[parameterIndex]);
    if (callingType !== undefined) {
        callingTypes.push(callingType);
    }
};
//# sourceMappingURL=index.js.map