"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixIncompletePropertyDeclarationTypes = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const creators_1 = require("../../../../mutations/creators");
const nodes_1 = require("../../../../shared/nodes");
const nodeTypes_1 = require("../../../../shared/nodeTypes");
const types_1 = require("../../../../shared/types");
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const fixIncompletePropertyDeclarationTypes = (request) => collectMutationsFromNodes_1.collectMutationsFromNodes(request, isPropertyDeclarationWithType, visitPropertyDeclaration);
exports.fixIncompletePropertyDeclarationTypes = fixIncompletePropertyDeclarationTypes;
const isPropertyDeclarationWithType = (node) => ts.isPropertyDeclaration(node) && nodeTypes_1.isNodeWithType(node);
const visitPropertyDeclaration = (node, request) => {
    // Collect types later assigned to the property, and types initially declared by or inferred on the property
    const assignedTypes = collectPropertyAssignedTypes(node, request);
    const declaredType = types_1.getTypeAtLocationIfNotError(request, node);
    if (declaredType === undefined) {
        return undefined;
    }
    // If the property already has a declared type, add assigned types to it if necessary
    if (nodeTypes_1.isNodeWithType(node)) {
        return creators_1.createTypeAdditionMutation(request, node, declaredType, assignedTypes);
    }
    // Since the parameter doesn't have its own type, give it one if necessary
    return creators_1.createTypeCreationMutation(request, node, declaredType, assignedTypes);
};
const collectPropertyAssignedTypes = (node, request) => {
    const assignedTypes = [];
    // If the property has an initial value, consider that an assignment
    if (node.initializer !== undefined) {
        const initializerType = types_1.getTypeAtLocationIfNotError(request, node.initializer);
        if (initializerType !== undefined) {
            assignedTypes.push(initializerType);
        }
    }
    // If the property is marked as readonly, don't bother checking for more types
    if (tsutils.isModifierFlagSet(node, ts.ModifierFlags.Readonly)) {
        return assignedTypes;
    }
    // Find everything else referencing the property
    const references = request.fileInfoCache.getNodeReferencesAsNodes(node);
    if (references !== undefined) {
        // For each referencing location, update types if the type is assigned to there
        for (const reference of references) {
            updateAssignedTypesForReference(node.parent, reference, assignedTypes, request);
        }
    }
    return assignedTypes;
};
/**
 * Adds missing types for a reference to a property.
 *
 * @param identifier   Node referencing the property.
 * @param assignedTypes   In-progress collection of types assigned to a property.
 * @param request   Metadata and settings to collect mutations in a file.
 */
const updateAssignedTypesForReference = (targetClass, identifier, assignedTypes, request) => {
    var _a;
    // In order to write a new type, the referencing node should be an identifier...
    if (!ts.isIdentifier(identifier)) {
        return;
    }
    // ...contained as a name inside a property access...
    const propertyAccess = identifier.parent;
    if (!ts.isPropertyAccessExpression(propertyAccess) || propertyAccess.name !== identifier) {
        return;
    }
    // ...contained as the left-hand side of an "=" binary expression...
    const binaryExpression = propertyAccess.parent;
    if (!nodes_1.isNodeAssigningBinaryExpression(binaryExpression) || binaryExpression.left !== propertyAccess) {
        return;
    }
    // ...and where the original property access expression refers to the target class
    // (this is important when multiple child classes of a single base class redeclare a member, such as React.Component's state)
    const assigneeType = types_1.getTypeAtLocationIfNotError(request, propertyAccess.expression);
    if (((_a = assigneeType === null || assigneeType === void 0 ? void 0 : assigneeType.getSymbol()) === null || _a === void 0 ? void 0 : _a.valueDeclaration) !== targetClass) {
        return;
    }
    // Mark the type of the right-hand side of the "=" expression as being assigned
    const assignmentType = types_1.getTypeAtLocationIfNotError(request, binaryExpression.right);
    if (assignmentType !== undefined) {
        assignedTypes.push(assignmentType);
    }
};
//# sourceMappingURL=index.js.map