"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectAllFunctionCallTypes = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const calls_1 = require("../../../../../shared/calls");
const nodeTypes_1 = require("../../../../../shared/nodeTypes");
const types_1 = require("../../../../../shared/types");
const getPropNodeFromReference_1 = require("../getPropNodeFromReference");
const collectAllFunctionCallTypes = (request, propsNode) => {
    const allFunctionCallTypes = new Map();
    for (const member of propsNode.members) {
        // Don't look at properties without obvious names and types explicitly set to Function
        if (!nodeTypes_1.isPropertySignatureWithStaticName(member) ||
            member.type === undefined ||
            !ts.isTypeReferenceNode(member.type) ||
            !ts.isIdentifier(member.type.typeName) ||
            member.type.typeName.text !== "Function") {
            continue;
        }
        collectFunctionCallsTypes(request, member, allFunctionCallTypes);
    }
    return allFunctionCallTypes;
};
exports.collectAllFunctionCallTypes = collectAllFunctionCallTypes;
const collectFunctionCallsTypes = (request, member, allFunctionCallTypes) => {
    // Find all references to the name of the type
    const references = request.fileInfoCache.getNodeReferencesAsNodes(member.name);
    if (references === undefined) {
        return;
    }
    const functionCallTypes = [];
    // For each reference, try to infer the type from its usage...
    for (const reference of references) {
        // (except for the original member we're looking around)
        if (reference === member || !tsutils.isExpression(reference)) {
            continue;
        }
        const call = getCallForReference(reference);
        if (call !== undefined) {
            collectFunctionCallTypes(request, functionCallTypes, call);
        }
    }
    allFunctionCallTypes.set(member, functionCallTypes);
};
const getCallForReference = (reference) => {
    reference = getPropNodeFromReference_1.getPropNodeFromReference(reference);
    return ts.isCallExpression(reference.parent) ? reference.parent : undefined;
};
const collectFunctionCallTypes = (request, functionCallTypes, call) => {
    // Case: the return value is directly passed to a function
    if (ts.isCallExpression(call.parent) || ts.isNewExpression(call.parent)) {
        const declaredTypes = calls_1.getDeclaredTypesOfArgument(request, call.parent, call);
        for (const declaredType of declaredTypes) {
            functionCallTypes.push({
                returnValue: declaredType,
            });
        }
    }
    // If the prop function is passed arguments, infer types from them
    if (call.arguments.length !== 0) {
        functionCallTypes.push({
            parameters: call.arguments.map((callArgument) => types_1.getTypeAtLocationIfNotError(request, callArgument)),
        });
    }
};
//# sourceMappingURL=collectAllFunctionCallTypes.js.map