"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createFunctionCallTypesMutation = void 0;
const automutate_1 = require("automutate");
const arrays_1 = require("../../../../../shared/arrays");
const createFunctionCallTypesMutation = (request, allFunctionCallTypes) => {
    const mutations = Array.from(allFunctionCallTypes).map(([member, functionCallTypes]) => {
        return createFunctionCallTypeMutation(request, member, functionCallTypes);
    });
    return mutations === undefined || mutations.length === 0 ? undefined : automutate_1.combineMutations(...mutations);
};
exports.createFunctionCallTypesMutation = createFunctionCallTypesMutation;
const createFunctionCallTypeMutation = (request, member, functionCallTypes) => {
    const combinedType = functionCallTypes.reduce((accum, functionCallType) => {
        return {
            parameters: combineParameters(request, accum.parameters, functionCallType.parameters),
            returnValue: arrays_1.collectOptionals(accum.returnValue, [functionCallType.returnValue]).filter(arrays_1.isNotUndefined),
        };
    }, {
        parameters: [],
        returnValue: [],
    });
    return {
        insertion: `${member.name.text}: ${printFunctionType(request, combinedType)}`,
        range: {
            begin: member.getStart(request.sourceFile),
            end: member.end,
        },
        type: "text-swap",
    };
};
const combineParameters = (request, previous, next) => {
    if (next === undefined) {
        return previous;
    }
    const combined = [];
    let i;
    for (i = 0; i < previous.length; i += 1) {
        combined.push([...previous[i]]);
        if (i < next.length && next[i] !== undefined) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            combined[i].push(next[i]);
        }
    }
    for (i; i < next.length; i += 1) {
        if (next[i] !== undefined) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            combined.push([next[i]]);
        }
    }
    return combined;
};
const printFunctionType = (request, combinedType) => {
    var _a, _b;
    return [
        "(",
        (_a = combinedType.parameters) === null || _a === void 0 ? void 0 : _a.map((parameter, index) => `arg${index}: ${request.services.printers.type(parameter)}`).join(", "),
        ") => ",
        ((_b = combinedType.returnValue) === null || _b === void 0 ? void 0 : _b.length) ? request.services.printers.type(combinedType.returnValue) : "void",
        ";",
    ].join("");
};
//# sourceMappingURL=createFunctionCallTypesMutation.js.map