"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getComponentAssignedTypesFromUsage = void 0;
const ts = __importStar(require("typescript"));
const names_1 = require("../../../../../shared/names");
const types_1 = require("../../../../../shared/types");
/**
 * Finds all assigned types for props in each JSX usage of a React component.
 */
const getComponentAssignedTypesFromUsage = (request, node) => {
    const references = getComponentReferences(request, node);
    if (references === undefined) {
        return undefined;
    }
    const assignedTypes = [];
    // For each referencing location, update types if the node is used as a component there
    for (const reference of references) {
        updateAssignedTypesForReference(reference, assignedTypes, request);
    }
    return assignedTypes;
};
exports.getComponentAssignedTypesFromUsage = getComponentAssignedTypesFromUsage;
const getComponentReferences = (request, node) => {
    const referencesNode = ts.isClassDeclaration(node) || ts.isFunctionDeclaration(node) ? node : node.parent;
    return request.fileInfoCache.getNodeReferencesAsNodes(referencesNode);
};
const updateAssignedTypesForReference = (identifier, componentAssignedTypes, request) => {
    // In order to assign props, the referencing node should be an identifier...
    if (!ts.isIdentifier(identifier)) {
        return;
    }
    // ...inside a starting JSX element
    const jsxElement = identifier.parent;
    if (!ts.isJsxSelfClosingElement(jsxElement) && !ts.isJsxOpeningElement(jsxElement)) {
        return;
    }
    // For each property passed in this element's attributes, grab its name and type into a map
    const assignedTypes = new Map();
    for (const property of jsxElement.attributes.properties) {
        // Ignore properties that aren't directly JSX attributes
        if (!ts.isJsxAttribute(property)) {
            continue;
        }
        // For now, ignore any property with a name that's not immediately convertable to a string
        const name = names_1.getStaticNameOfProperty(property.name);
        if (name === undefined) {
            continue;
        }
        // TypeScript stores the type of the property's value on the property itself
        const propertyType = types_1.getTypeAtLocationIfNotError(request, property);
        if (propertyType !== undefined) {
            assignedTypes.set(name, propertyType);
        }
    }
    componentAssignedTypes.push(assignedTypes);
};
//# sourceMappingURL=getComponentAssignedTypesFromUsage.js.map