"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixReactPropsFromLaterAssignments = void 0;
const expansionMutations_1 = require("../../../../../mutations/expansions/expansionMutations");
const collectMutationsFromNodes_1 = require("../../../../collectMutationsFromNodes");
const isReactComponentNode_1 = require("../reactFiltering/isReactComponentNode");
const getComponentAssignedTypesFromUsage_1 = require("./getComponentAssignedTypesFromUsage");
const getComponentPropsNode_1 = require("../getComponentPropsNode");
/**
 * Expands the existing props type for a component from its external JSX-style declarations.
 */
const fixReactPropsFromLaterAssignments = (request) => {
    return collectMutationsFromNodes_1.collectMutationsFromNodes(request, isReactComponentNode_1.isReactComponentNode, visitReactComponentNode);
};
exports.fixReactPropsFromLaterAssignments = fixReactPropsFromLaterAssignments;
const visitReactComponentNode = (node, request) => {
    // Grab the node used to declare the node's props type, if it exists
    const propsNode = getComponentPropsNode_1.getComponentPropsNode(request, node);
    if (propsNode === undefined) {
        return undefined;
    }
    // Find all types of props later passed to the node
    const componentAssignedTypes = getComponentAssignedTypesFromUsage_1.getComponentAssignedTypesFromUsage(request, node);
    if (componentAssignedTypes === undefined || componentAssignedTypes.length === 0) {
        return undefined;
    }
    return expansionMutations_1.createTypeExpansionMutation(request, propsNode, componentAssignedTypes);
};
//# sourceMappingURL=index.js.map