"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixReactPropsFromPropTypes = void 0;
const automutate_1 = require("automutate");
const ts = __importStar(require("typescript"));
const enums_1 = require("../../../../../options/enums");
const nodes_1 = require("../../../../../shared/nodes");
const newlines_1 = require("../../../../../shared/printing/newlines");
const collectMutationsFromNodes_1 = require("../../../../collectMutationsFromNodes");
const isReactComponentNode_1 = require("../reactFiltering/isReactComponentNode");
const createInterfaceUsageMutation_1 = require("./annotation/createInterfaceUsageMutation");
const createInterfaceFromPropTypes_1 = require("./propTypes/createInterfaceFromPropTypes");
const getPropTypesValue_1 = require("./propTypes/getPropTypesValue");
/**
 * Creates an initial props type for a component from its PropTypes declaration.
 */
const fixReactPropsFromPropTypes = (request) => {
    if (request.options.hints.react.propTypes === enums_1.ReactPropTypesHint.Ignore) {
        return [];
    }
    return collectMutationsFromNodes_1.collectMutationsFromNodes(request, isReactComponentNode_1.isReactComponentNode, visitReactComponentNode);
};
exports.fixReactPropsFromPropTypes = fixReactPropsFromPropTypes;
const visitReactComponentNode = (node, request) => {
    // If the node is a class declaration, don't bother with prop types if it already declares a React.Component template
    if (ts.isClassDeclaration(node)) {
        const extendsType = nodes_1.getClassExtendsType(node);
        if ((extendsType === null || extendsType === void 0 ? void 0 : extendsType.typeArguments) !== undefined && extendsType.typeArguments.length > 0) {
            return undefined;
        }
    }
    // Try to find a static `propTypes` member to indicate the interface
    // If it doesn't exist, we can't infer anything about the component here, so we bail out
    const propTypes = getPropTypesValue_1.getPropTypesValue(node);
    if (propTypes === undefined) {
        return undefined;
    }
    // Since we did find the propTypes object, we can generate an interface from its members
    const { interfaceName, interfaceNode } = createInterfaceFromPropTypes_1.createInterfaceFromPropTypes(request, node, propTypes);
    // That interface will be injected with blank lines around it just before the component
    const mutations = [createInterfaceCreationMutation(request, node, interfaceNode)];
    // We'll also annotate the component with a type declaration to use the new prop type
    const usage = createInterfaceUsageMutation_1.createInterfaceUsageMutation(node, interfaceName);
    if (usage !== undefined) {
        mutations.push(usage);
    }
    return automutate_1.combineMutations(...mutations);
};
const createInterfaceCreationMutation = (request, node, interfaceNode) => {
    const endline = newlines_1.printNewLine(request.options.compilerOptions);
    const interfaceNodeText = request.services.printers.node(interfaceNode);
    return {
        insertion: [endline, endline, interfaceNodeText, endline].join(""),
        range: {
            begin: node.pos,
        },
        type: "text-insert",
    };
};
//# sourceMappingURL=index.js.map