"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropTypesValue = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
/**
 * @returns Whether a node is a `propTypes` class member with an object literal value.
 */
const getStaticPropTypes = (node) => ts.isPropertyDeclaration(node) &&
    tsutils.hasModifier(node.modifiers, ts.SyntaxKind.StaticKeyword) &&
    ts.isIdentifier(node.name) &&
    node.name.text === "propTypes" &&
    node.initializer !== undefined &&
    ts.isObjectLiteralExpression(node.initializer);
const getPropTypesChildFromParent = (parent, className) => {
    let result;
    parent.forEachChild((child) => {
        const propTypes = isPropTypesStatement(child, className);
        if (propTypes !== undefined) {
            result = propTypes;
            return true;
        }
        return false;
    });
    return result;
};
/**
 * @returns Object literal `propTypes` assigned to the class, if it exists in a sibling property setter.
 */
const isPropTypesStatement = (node, className) => {
    if (!ts.isExpressionStatement(node) || !ts.isBinaryExpression(node.expression)) {
        return undefined;
    }
    const { left, right } = node.expression;
    if (!ts.isPropertyAccessExpression(left) ||
        !ts.isIdentifier(left.expression) ||
        left.expression.text !== className ||
        !ts.isIdentifier(left.name) ||
        left.name.text !== "propTypes" ||
        !ts.isObjectLiteralExpression(right)) {
        return undefined;
    }
    return right;
};
/**
 * Finds the equivalent `propTypes` object literal for a class, if it exists.
 *
 * @todo
 * This assumes an object literal (`{ ... }`) with all inline members.
 * It doesn't yet handle shared variable references or `...` spread operations.
 */
const getPropTypesValue = (node) => {
    // If the component's parent declares a prop types for it, use it
    if (node.name !== undefined) {
        const propTypesStatement = getPropTypesChildFromParent(node.parent, node.name.text);
        if (propTypesStatement !== undefined) {
            return propTypesStatement;
        }
    }
    // If the component is a class that declares its own static prop types, use it
    if (ts.isClassDeclaration(node)) {
        const staticPropTypes = node.members.find(getStaticPropTypes);
        if (staticPropTypes !== undefined) {
            return staticPropTypes.initializer;
        }
    }
    // Since none of the above worked out, assume no prop types are declared
    return undefined;
};
exports.getPropTypesValue = getPropTypesValue;
//# sourceMappingURL=getPropTypesValue.js.map