"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropTypesMember = void 0;
const ts = __importStar(require("typescript"));
/**
 * Selects the relevant member nodes for a PropTypes member expression.
 *
 * @param node
 * PropTypes assignment in an object literal, as one of:
 * `PropTypes.number`
 * `PropTypes.number.isRequired`
 * `PropTypes.shape({})`
 * `PropTypes.shape({}).isRequired`
 */
const getPropTypesMember = (node) => {
    if (ts.isCallExpression(node)) {
        return getPropTypesMemberFromCallExpression(node);
    }
    if (ts.isPropertyAccessExpression(node)) {
        return getPropTypesMemberFromPropertyAccessExpression(node);
    }
    return undefined;
};
exports.getPropTypesMember = getPropTypesMember;
/**
 * @remarks
 * Must be like `PropTypes.shape({})`, as `.isRequired` would make this a property access expression.
 */
const getPropTypesMemberFromCallExpression = (node) => {
    const accessNode = node.expression;
    if (!ts.isPropertyAccessExpression(accessNode)) {
        return undefined;
    }
    const nameNode = accessNode.name;
    return { accessNode, nameNode };
};
/**
 * @remarks
 * Must be like `PropTypes.number`, `PropTypes.number.isRequired`, or `PropTypes.shape({}).isRequired`.
 */
const getPropTypesMemberFromPropertyAccessExpression = (node) => {
    const isRequired = node.name.text === "isRequired" ? node.name : undefined;
    return isRequired === undefined
        ? getPropTypesMemberFromPropertyAccessExpressionOptional(node)
        : getPropTypesMemberFromPropertyAccessExpressionRequired(node, isRequired);
};
/**
 * @remarks
 * Must be like `PropTypes.number`.
 */
const getPropTypesMemberFromPropertyAccessExpressionOptional = (node) => {
    return { accessNode: node, nameNode: node.name };
};
/**
 * @remarks
 * Must be like `PropTypes.shape({}).isRequired` or `PropTypes.number.isRequired`.
 */
const getPropTypesMemberFromPropertyAccessExpressionRequired = (node, isRequired) => {
    const { expression } = node;
    if (ts.isCallExpression(expression)) {
        const callExpressionTypesMember = getPropTypesMemberFromCallExpression(expression);
        if (callExpressionTypesMember === undefined) {
            return undefined;
        }
        return Object.assign(Object.assign({}, callExpressionTypesMember), { isRequired });
    }
    if (!ts.isPropertyAccessExpression(expression)) {
        return undefined;
    }
    return {
        accessNode: expression,
        isRequired,
        nameNode: expression.name,
    };
};
//# sourceMappingURL=propTypesExtraction.js.map