"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPropTypesTransform = void 0;
const ts = __importStar(require("typescript"));
const transforms_1 = require("../../../../../../shared/transforms");
const propTypesExtraction_1 = require("./propTypesExtraction");
const propTypesProperties_1 = require("./propTypesProperties");
const createPropTypesTransform = (request, { accessNode, nameNode }) => {
    switch (nameNode.text) {
        case "array":
            return ts.factory.createArrayTypeNode(ts.factory.createKeywordTypeNode(ts.SyntaxKind.AnyKeyword));
        case "arrayOf":
            return createArrayOfTransform(request, accessNode);
        case "bool":
            return ts.factory.createKeywordTypeNode(ts.SyntaxKind.BooleanKeyword);
        case "func":
            return ts.factory.createTypeReferenceNode(ts.factory.createIdentifier("Function"), undefined);
        case "element":
            return ts.factory.createTypeReferenceNode(ts.factory.createIdentifier("React.ReactElement"), undefined);
        case "instanceOf":
            return createInstanceOfTransform(accessNode);
        case "number":
            return ts.factory.createKeywordTypeNode(ts.SyntaxKind.NumberKeyword);
        case "node":
            return ts.factory.createTypeReferenceNode(ts.factory.createIdentifier("React.ReactNode"), undefined);
        case "object":
            return ts.factory.createKeywordTypeNode(ts.SyntaxKind.ObjectKeyword);
        case "oneOf":
            return createOneOfTransform(accessNode);
        case "oneOfType":
            return createOneOfTypeTransform(request, accessNode);
        case "shape":
            return createShapeTransform(request, accessNode);
        case "string":
            return ts.factory.createKeywordTypeNode(ts.SyntaxKind.StringKeyword);
        case "symbol":
            return ts.factory.createKeywordTypeNode(ts.SyntaxKind.SymbolKeyword);
    }
    return undefined;
};
exports.createPropTypesTransform = createPropTypesTransform;
const createArrayOfTransform = (request, accessNode) => {
    if (!ts.isCallExpression(accessNode.parent) || accessNode.parent.arguments.length !== 1) {
        return undefined;
    }
    const memberTypeNode = propTypesExtraction_1.getPropTypesMember(accessNode.parent.arguments[0]);
    if (memberTypeNode === undefined) {
        return undefined;
    }
    const innerTransform = exports.createPropTypesTransform(request, memberTypeNode);
    if (innerTransform === undefined) {
        return undefined;
    }
    return ts.factory.createArrayTypeNode(innerTransform);
};
const createInstanceOfTransform = (accessNode) => {
    if (!ts.isCallExpression(accessNode.parent) || accessNode.parent.arguments.length !== 1) {
        return undefined;
    }
    const className = accessNode.parent.arguments[0];
    if (!ts.isIdentifier(className)) {
        return undefined;
    }
    return ts.factory.createTypeReferenceNode(ts.factory.createIdentifier(className.text), undefined);
};
const createOneOfTransform = (accessNode) => {
    if (!ts.isCallExpression(accessNode.parent) || accessNode.parent.arguments.length !== 1) {
        return undefined;
    }
    const allowedItems = accessNode.parent.arguments[0];
    if (!ts.isArrayLiteralExpression(allowedItems)) {
        return undefined;
    }
    const allowedTypes = allowedItems.elements
        .map(transforms_1.transformLiteralToTypeLiteralNode)
        .filter((typeNode) => typeNode !== undefined);
    if (allowedTypes.length === 0) {
        return undefined;
    }
    return ts.factory.createUnionTypeNode(allowedTypes);
};
const createOneOfTypeTransform = (request, accessNode) => {
    if (!ts.isCallExpression(accessNode.parent) || accessNode.parent.arguments.length !== 1) {
        return undefined;
    }
    const allowedItems = accessNode.parent.arguments[0];
    if (!ts.isArrayLiteralExpression(allowedItems)) {
        return undefined;
    }
    const allowedTypes = allowedItems.elements
        .map((element) => {
        if (!ts.isPropertyAccessExpression(element) || !ts.isIdentifier(element.name)) {
            return undefined;
        }
        return exports.createPropTypesTransform(request, {
            accessNode: element,
            nameNode: element.name,
        });
    })
        .filter((typeName) => typeName !== undefined);
    if (allowedTypes.length === 0) {
        return undefined;
    }
    return ts.factory.createUnionTypeNode(allowedTypes);
};
const createShapeTransform = (request, accessNode) => {
    if (!ts.isCallExpression(accessNode.parent) || accessNode.parent.arguments.length !== 1) {
        return undefined;
    }
    const shape = accessNode.parent.arguments[0];
    // Todo: handle shared variables and `...` object spreads
    if (!ts.isObjectLiteralExpression(shape)) {
        return undefined;
    }
    const members = [];
    for (const rawProperty of shape.properties) {
        const member = propTypesProperties_1.createPropTypesProperty(request, rawProperty);
        if (member !== undefined) {
            members.push(member);
        }
    }
    return ts.factory.createTypeLiteralNode(members);
};
//# sourceMappingURL=propTypesTransforms.js.map