"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPropsUsageTypes = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const calls_1 = require("../../../../../shared/calls");
const nodes_1 = require("../../../../../shared/nodes");
const nodeTypes_1 = require("../../../../../shared/nodeTypes");
const types_1 = require("../../../../../shared/types");
const getComponentPropsNode_1 = require("../getComponentPropsNode");
const getPropNodeFromReference_1 = require("../getPropNodeFromReference");
const getReactComponentNode_1 = require("../reactFiltering/getReactComponentNode");
const getPropsUsageTypes = (request, componentNode, propsNode) => {
    const allAssignedTypes = [];
    for (const member of propsNode.members) {
        if (nodeTypes_1.isPropertySignatureWithStaticName(member)) {
            updateAssignedTypesForReferences(request, member, componentNode, member.name, new Set(), allAssignedTypes);
        }
    }
    return allAssignedTypes;
};
exports.getPropsUsageTypes = getPropsUsageTypes;
const updateAssignedTypesForReferences = (request, member, componentNode, start, seenNodes, allAssignedTypes) => {
    var _a;
    // Don't repeatedly bounce back and forth between references
    if (seenNodes.has(start)) {
        return;
    }
    seenNodes.add(start);
    // Find all references to the name of the type within the component
    const references = (_a = request.fileInfoCache
        .getNodeReferencesAsNodes(start)) === null || _a === void 0 ? void 0 : _a.filter((reference) => nodes_1.isNodeWithinNode(request.sourceFile, reference, componentNode));
    if (references === undefined || references.length === 0) {
        return;
    }
    // For each reference, try to infer the type from its usage...
    for (const reference of references) {
        // (except for the original member we're looking around)
        if (reference === member) {
            continue;
        }
        // If the reference is an indirect storage, such as a variable, recurse on *its* references
        if (!tsutils.isExpression(reference)) {
            updateAssignedTypesForReferences(request, member, componentNode, reference, seenNodes, allAssignedTypes);
            continue;
        }
        const propUsage = getPropNodeFromReference_1.getPropNodeFromReference(reference);
        // Case: used as the value in a JSX attribute
        if (ts.isJsxExpression(propUsage.parent)) {
            // Use the type value for the declaration of that attribute (its own prop)
            const attributePropType = getAttributePropType(request, propUsage.parent.parent);
            if (attributePropType !== undefined) {
                allAssignedTypes.push(new Map([[member.name.text, attributePropType]]));
            }
            continue;
        }
        // Case: passed as an argument to a function
        if (ts.isCallExpression(propUsage.parent) || ts.isNewExpression(propUsage.parent)) {
            const declaredTypes = calls_1.getDeclaredTypesOfArgument(request, propUsage.parent, propUsage);
            for (const declaredType of declaredTypes) {
                allAssignedTypes.push(new Map([[member.name.text, declaredType]]));
            }
        }
    }
};
const getAttributePropType = (request, attribute) => {
    var _a;
    if (!ts.isJsxAttribute(attribute) || !nodeTypes_1.isNodeWithIdentifierName(attribute)) {
        return undefined;
    }
    // Find the corresponding declaration or (containing variable) for the node's backing React component
    // In theory we could try to get the symbol or type at the location,
    // but in practice it seems that always resolves to the (potentially incomplete) provided type
    const { tagName } = attribute.parent.parent;
    const tagDeclaration = (_a = types_1.getSymbolAtLocationIfNotError(request, tagName)) === null || _a === void 0 ? void 0 : _a.valueDeclaration;
    if (tagDeclaration === undefined) {
        return undefined;
    }
    // Get the React component associated with the declaration
    const tagComponent = getReactComponentNode_1.getReactComponentNode(tagDeclaration);
    if (tagComponent === undefined) {
        return undefined;
    }
    // Find the props node declaration for that component,
    // and within that, the specific prop declaration
    const tagProps = getComponentPropsNode_1.getComponentPropsNode(request, tagComponent);
    const propDeclaration = tagProps === null || tagProps === void 0 ? void 0 : tagProps.members.find((member) => nodeTypes_1.isNodeWithIdentifierName(member) && member.name.text === attribute.name.text);
    return types_1.getTypeAtLocationIfNotError(request, propDeclaration);
};
//# sourceMappingURL=getPropsUsageTypes.js.map