"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getComponentPropsNode = void 0;
const ts = __importStar(require("typescript"));
const nodes_1 = require("../../../../shared/nodes");
const types_1 = require("../../../../shared/types");
/**
 * Finds the corresponding interface or type literal that declares a component's props type, if it exists.
 */
const getComponentPropsNode = (request, node) => {
    return ts.isClassDeclaration(node) || ts.isClassExpression(node)
        ? getClassComponentPropsNode(request, node)
        : getFunctionComponentPropsNode(request, node);
};
exports.getComponentPropsNode = getComponentPropsNode;
const getClassComponentPropsNode = (request, node) => {
    const extendsType = nodes_1.getClassExtendsType(node);
    if ((extendsType === null || extendsType === void 0 ? void 0 : extendsType.typeArguments) === undefined || extendsType.typeArguments.length === 0) {
        return undefined;
    }
    const [rawPropsNode] = extendsType.typeArguments;
    const propsNodeType = types_1.getTypeAtLocationIfNotError(request, rawPropsNode);
    const propsNodeSymbol = propsNodeType === null || propsNodeType === void 0 ? void 0 : propsNodeType.getSymbol();
    if (propsNodeSymbol === undefined) {
        return undefined;
    }
    const symbolDeclarations = propsNodeSymbol.getDeclarations();
    const declaration = symbolDeclarations === undefined || symbolDeclarations.length === 0 ? undefined : symbolDeclarations[0];
    return declaration !== undefined && isReactComponentPropsNode(declaration) ? declaration : undefined;
};
const getFunctionComponentPropsNode = (request, node) => {
    var _a, _b;
    // If the node takes multiple parameters, we assume it's not an FC
    // TODO: this might not hold true for refs or other fancy things...
    const { parameters } = node;
    if (parameters.length !== 1) {
        return undefined;
    }
    // Try to get the first backing type declaration for the single parameter
    const [parameter] = parameters;
    const symbol = (_a = types_1.getTypeAtLocationIfNotError(request, parameter)) === null || _a === void 0 ? void 0 : _a.getSymbol();
    if (!((_b = symbol === null || symbol === void 0 ? void 0 : symbol.declarations) === null || _b === void 0 ? void 0 : _b.length)) {
        return undefined;
    }
    const [declaration] = symbol.declarations;
    if (isReactComponentPropsNode(declaration)) {
        return declaration;
    }
    return undefined;
};
const isReactComponentPropsNode = (node) => ts.isInterfaceDeclaration(node) || ts.isTypeLiteralNode(node);
//# sourceMappingURL=getComponentPropsNode.js.map