"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixIncompleteVariableTypes = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const creators_1 = require("../../../../mutations/creators");
const nodeTypes_1 = require("../../../../shared/nodeTypes");
const references_1 = require("../../../../shared/references");
const types_1 = require("../../../../shared/types");
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const fixIncompleteVariableTypes = (request) => collectMutationsFromNodes_1.collectMutationsFromNodes(request, isNodeVariableDeclarationWithType, visitVariableDeclaration);
exports.fixIncompleteVariableTypes = fixIncompleteVariableTypes;
const isNodeVariableDeclarationWithType = (node) => ts.isVariableDeclaration(node) && nodeTypes_1.isNodeWithType(node);
const visitVariableDeclaration = (node, request) => {
    // Collect types later assigned to the variable, and types initially declared by or inferred on the variable
    const assignedTypes = collectVariableAssignedTypes(node, request);
    const declaredType = types_1.getTypeAtLocationIfNotError(request, node);
    if (declaredType === undefined) {
        return undefined;
    }
    // If the variable already has a declared type, add assigned types to it if necessary
    if (nodeTypes_1.isNodeWithType(node)) {
        return creators_1.createTypeAdditionMutation(request, node, declaredType, assignedTypes);
    }
    // Since the node's missing type isn't inferrable, try our best to give it one
    return creators_1.createTypeCreationMutation(request, node, declaredType, assignedTypes);
};
/**
 * Finds types later assigned to a variable declaration.
 *
 * @param node   Node to collect types from.
 * @param request   Metadata and settings to collect mutations in a file.
 * @returns Types assigned to the node in the file.
 */
const collectVariableAssignedTypes = (node, request) => {
    const assignedTypes = [];
    // If the variable has an initial value, consider that an assignment
    if (node.initializer !== undefined) {
        const initializerType = types_1.getTypeAtLocationIfNotError(request, node.initializer);
        if (initializerType !== undefined) {
            assignedTypes.push(initializerType);
        }
    }
    // If the variable is anonymous or marked as readonly, don't bother checking for more types
    if (!ts.isIdentifier(node.name) || tsutils.isModifierFlagSet(node, ts.ModifierFlags.Readonly)) {
        return assignedTypes;
    }
    // Collect all places in the file where the variable is later referenced
    const variableInfo = request.fileInfoCache.getVariableUsage().get(node.name);
    if (variableInfo === undefined) {
        return assignedTypes;
    }
    for (const use of variableInfo.uses) {
        const useIdentifier = use.location;
        const useExpression = useIdentifier.parent;
        // Ignore the node usage if it's inside an ignored node
        if (references_1.isNodeFilteredOut(request.filteredNodes, use.location)) {
            continue;
        }
        // We only care about binary expressions that assign a type to the variable
        if (!ts.isBinaryExpression(useExpression) ||
            useExpression.operatorToken.getText(request.sourceFile) !== "=" ||
            useExpression.left !== useIdentifier) {
            continue;
        }
        // Grab the new type being assigned to the node
        const assignmentType = types_1.getTypeAtLocationIfNotError(request, useExpression.right);
        if (assignmentType !== undefined) {
            assignedTypes.push(assignmentType);
        }
    }
    return assignedTypes;
};
//# sourceMappingURL=index.js.map