"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixStrictNonNullAssertionCallExpressions = void 0;
const automutate_1 = require("automutate");
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const flags_1 = require("../../../../mutations/collecting/flags");
const createNonNullAssertion_1 = require("../../../../mutations/typeMutating/createNonNullAssertion");
const functionTypes_1 = require("../../../../shared/functionTypes");
const nodes_1 = require("../../../../shared/nodes");
const nodeTypes_1 = require("../../../../shared/nodeTypes");
const types_1 = require("../../../../shared/types");
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const fixStrictNonNullAssertionCallExpressions = (request) => {
    return collectMutationsFromNodes_1.collectMutationsFromNodes(request, isVisitableCallExpression, visitCallExpression);
};
exports.fixStrictNonNullAssertionCallExpressions = fixStrictNonNullAssertionCallExpressions;
const isVisitableCallExpression = (node) => ts.isCallExpression(node) &&
    // We can quickly ignore any calls without arguments
    node.arguments.length !== 0;
const visitCallExpression = (node, request) => {
    // Collect the declared type of the function-like being called
    const functionLikeValueDeclaration = functionTypes_1.getValueDeclarationOfFunction(request, node.expression);
    if (functionLikeValueDeclaration === undefined) {
        return undefined;
    }
    // Collect mutations for each argument as needed
    const argumentMutations = collectArgumentMutations(request, node, functionLikeValueDeclaration);
    if (argumentMutations.length === 0) {
        return undefined;
    }
    return automutate_1.combineMutations(...argumentMutations);
};
const collectArgumentMutations = (request, callingNode, functionLikeValueDeclaration) => {
    const mutations = [];
    const visitableArguments = Math.min(callingNode.arguments.length, functionLikeValueDeclaration.parameters.length);
    // Check the types of each argument being passed in against the declared parameter type
    for (let i = 0; i < visitableArguments; i += 1) {
        // We can ignore parameters that are 'any'
        const typeOfParameter = types_1.getTypeAtLocationIfNotError(request, functionLikeValueDeclaration.parameters[i]);
        if (typeOfParameter === undefined || flags_1.isTypeFlagSetRecursively(typeOfParameter, ts.TypeFlags.Any)) {
            continue;
        }
        const typeOfArgument = types_1.getTypeAtLocationIfNotError(request, callingNode.arguments[i]);
        // If either null or undefined is missing in the argument, we'll need a ! mutation
        if (typeOfArgument !== undefined && nodeTypes_1.isNullOrUndefinedMissingBetween(typeOfArgument, typeOfParameter)) {
            mutations.push(collectArgumentMutation(request, callingNode.arguments[i]));
        }
    }
    return mutations;
};
const collectArgumentMutation = (request, callingArgument) => {
    // If the argument is a variable declared in the parent function, add the ! to the variable
    if (ts.isIdentifier(callingArgument)) {
        const declaringVariableInitializer = nodes_1.getVariableInitializerForExpression(request, callingArgument, nodes_1.getParentOfKind(callingArgument, isFunctionBodyOrBlock));
        if (declaringVariableInitializer !== undefined) {
            return createNonNullAssertion_1.createNonNullAssertion(request, declaringVariableInitializer);
        }
    }
    // Otherwise add the ! at the calling site's argument
    return createNonNullAssertion_1.createNonNullAssertion(request, callingArgument);
};
const isFunctionBodyOrBlock = (node) => tsutils.isFunctionWithBody(node) || ts.isBlock(node) || ts.isSourceFile(node);
//# sourceMappingURL=index.js.map