"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fixStrictNonNullAssertionReturnTypes = void 0;
const automutate_1 = require("automutate");
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const flags_1 = require("../../../../mutations/collecting/flags");
const createNonNullAssertion_1 = require("../../../../mutations/typeMutating/createNonNullAssertion");
const nodes_1 = require("../../../../shared/nodes");
const nodeTypes_1 = require("../../../../shared/nodeTypes");
const types_1 = require("../../../../shared/types");
const collectMutationsFromNodes_1 = require("../../../collectMutationsFromNodes");
const collectReturningNodeExpressions_1 = require("./collectReturningNodeExpressions");
const fixStrictNonNullAssertionReturnTypes = (request) => collectMutationsFromNodes_1.collectMutationsFromNodes(request, isNodeVisitableFunctionLikeDeclaration, visitFunctionWithBody);
exports.fixStrictNonNullAssertionReturnTypes = fixStrictNonNullAssertionReturnTypes;
const isNodeVisitableFunctionLikeDeclaration = (node) => tsutils.isFunctionWithBody(node) &&
    // If the node has an implicit return type, we don't need to change anything
    nodeTypes_1.isNodeWithType(node);
const visitFunctionWithBody = (node, request) => {
    // Collect the type initially declared as returned and whether it contains null and/or undefined
    const declaredType = types_1.getTypeAtLocationIfNotError(request, node.type);
    // If the node's explicit return type contains 'any', we can't infer anything
    if (declaredType === undefined || flags_1.isTypeFlagSetRecursively(declaredType, ts.TypeFlags.Any)) {
        return undefined;
    }
    // If the type already has both null or undefined, all is well; rejoice!
    const returningNull = flags_1.isTypeFlagSetRecursively(declaredType, ts.TypeFlags.Null);
    const returningUndefined = flags_1.isTypeFlagSetRecursively(declaredType, ts.TypeFlags.Undefined);
    if (returningNull && returningUndefined) {
        return undefined;
    }
    // From now on, we only care about the two types of strict values that could be returned
    const missingReturnTypes = (returningNull ? 0 : ts.TypeFlags.Null) | (returningUndefined ? 0 : ts.TypeFlags.Undefined);
    // Collect ! additions for the return types of nodes in the function
    const returningNodeExpressions = collectReturningNodeExpressions_1.collectReturningNodeExpressions(node);
    const mutations = collectNonNullMutations(request, node, missingReturnTypes, returningNodeExpressions);
    return mutations.length === 0 ? undefined : automutate_1.combineMutations(...mutations);
};
const collectNonNullMutations = (request, functionLike, missingReturnTypes, expressions) => {
    const mutations = [];
    for (const expression of expressions) {
        // If the expression doesn't return a type missing from the return, it's already safe
        const expressionType = types_1.getTypeAtLocationIfNotError(request, expression);
        if (expressionType === undefined || !flags_1.isTypeFlagSetRecursively(expressionType, missingReturnTypes)) {
            continue;
        }
        // If the expression is an variable declared in the parent function, add the ! to the variable
        if (ts.isIdentifier(expression)) {
            const declaringVariableInitializer = nodes_1.getVariableInitializerForExpression(request, expression, functionLike);
            if (declaringVariableInitializer !== undefined) {
                mutations.push(createNonNullAssertion_1.createNonNullAssertion(request, declaringVariableInitializer));
                continue;
            }
        }
        // Otherwise, add it at the end of the expression
        mutations.push(createNonNullAssertion_1.createNonNullAssertion(request, expression));
    }
    return mutations;
};
//# sourceMappingURL=index.js.map