"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fillOutRawOptions = void 0;
const arrays_1 = require("../shared/arrays");
const enums_1 = require("./enums");
const collectAddedMutators_1 = require("./parsing/collectAddedMutators");
const collectFileOptions_1 = require("./parsing/collectFileOptions");
const collectNoImplicitAny_1 = require("./parsing/collectNoImplicitAny");
const collectNoImplicitThis_1 = require("./parsing/collectNoImplicitThis");
const collectPackageOptions_1 = require("./parsing/collectPackageOptions");
const collectStrictNullChecks_1 = require("./parsing/collectStrictNullChecks");
/**
 * Combines Node and CLi argument options with project and file metadata into TypeStat options.
 *
 * @returns Parsed TypeStat options, or a string for an error complaint.
 */
const fillOutRawOptions = ({ compilerOptions, cwd, fileNames, output, projectPath, rawOptions, }) => {
    var _a, _b, _c, _d, _e, _f;
    const rawOptionTypes = rawOptions.types === undefined ? {} : rawOptions.types;
    const noImplicitAny = collectNoImplicitAny_1.collectNoImplicitAny(compilerOptions, rawOptions);
    const noImplicitThis = collectNoImplicitThis_1.collectNoImplicitThis(compilerOptions, rawOptions);
    const { compilerStrictNullChecks, typeStrictNullChecks } = collectStrictNullChecks_1.collectStrictNullChecks(compilerOptions, rawOptionTypes);
    const packageOptions = collectPackageOptions_1.collectPackageOptions(cwd, rawOptions);
    const shell = [];
    if (rawOptions.postProcess !== undefined && rawOptions.postProcess.shell !== undefined) {
        shell.push(...rawOptions.postProcess.shell);
    }
    return {
        compilerOptions: Object.assign(Object.assign({}, compilerOptions), { noImplicitAny,
            noImplicitThis, strictNullChecks: compilerStrictNullChecks }),
        fileNames,
        files: collectFileOptions_1.collectFileOptions(rawOptions),
        filters: arrays_1.collectOptionals(rawOptions.filters),
        fixes: Object.assign({ importExtensions: false, incompleteTypes: false, missingProperties: false, noImplicitAny: false, noImplicitThis: false, noInferableTypes: false, strictNonNullAssertions: false }, rawOptions.fixes),
        hints: {
            react: {
                propTypes: (_c = (_b = (_a = rawOptions.hints) === null || _a === void 0 ? void 0 : _a.react) === null || _b === void 0 ? void 0 : _b.propTypes) !== null && _c !== void 0 ? _c : enums_1.ReactPropTypesHint.WhenRequired,
                propTypesOptionality: (_f = (_e = (_d = rawOptions.hints) === null || _d === void 0 ? void 0 : _d.react) === null || _e === void 0 ? void 0 : _e.propTypesOptionality) !== null && _f !== void 0 ? _f : enums_1.ReactPropTypesOptionality.AsWritten,
            },
        },
        mutators: collectAddedMutators_1.collectAddedMutators(rawOptions, packageOptions.directory, output),
        output,
        package: packageOptions,
        postProcess: { shell },
        projectPath,
        types: {
            strictNullChecks: typeStrictNullChecks,
        },
    };
};
exports.fillOutRawOptions = fillOutRawOptions;
//# sourceMappingURL=fillOutRawOptions.js.map