"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadOptions = void 0;
const path = __importStar(require("path"));
const glob_1 = require("../shared/glob");
const paths_1 = require("../shared/paths");
const fillOutRawOptions_1 = require("./fillOutRawOptions");
const findRawOptions_1 = require("./findRawOptions");
const optionVerification_1 = require("./optionVerification");
const parseRawCompilerOptions_1 = require("./parseRawCompilerOptions");
/**
 * Reads TypeStat options using a config path.
 *
 * @param argv   Root arguments passed to TypeStat.
 * @param output   Wraps process and logfile output.
 * @returns Promise for filled-out TypeStat options, or a string complaint from failing to make them.
 */
const loadOptions = (argv, output) => __awaiter(void 0, void 0, void 0, function* () {
    if (argv.config === undefined) {
        return "-c/--config file must be provided.";
    }
    const cwd = process.cwd();
    const foundRawOptions = yield findRawOptions_1.findRawOptions(cwd, argv.config);
    if (typeof foundRawOptions === "string") {
        return foundRawOptions;
    }
    const { allRawOptions, filePath } = foundRawOptions;
    const results = [];
    // Fill out each option in the config with its own separate settings
    // (or return the first failure string describing which one is incorrect)
    for (let i = 0; i < allRawOptions.length; i += 1) {
        const rawOptions = allRawOptions[i];
        const projectPath = getProjectPath(cwd, filePath, rawOptions);
        const [compilerOptions, fileSearch] = yield Promise.all([
            parseRawCompilerOptions_1.parseRawCompilerOptions(projectPath),
            collectFileNames(argv, cwd, rawOptions),
        ]);
        if (fileSearch === undefined) {
            continue;
        }
        const [fileGlobs, fileNames] = fileSearch;
        const implicitNodeModulesInclude = implicitNodeModulesIncluded(fileGlobs, fileNames);
        if (implicitNodeModulesInclude) {
            return `At least one path including node_modules was included implicitly: '${implicitNodeModulesInclude}'. Either adjust TypeStat's included files to not include node_modules (recommended) or explicitly include node_modules/ (not recommended).`;
        }
        const filledOutOptions = optionVerification_1.findComplaintForOptions(fillOutRawOptions_1.fillOutRawOptions({ argv, compilerOptions, cwd, fileNames, output, projectPath, rawOptions }));
        if (typeof filledOutOptions === "string") {
            return `Invalid options at index ${i}: ${filledOutOptions}`;
        }
        results.push(filledOutOptions);
    }
    return results;
});
exports.loadOptions = loadOptions;
const getProjectPath = (cwd, filePath, rawOptions) => {
    // If the TypeStat configuration file has a projectPath field, use that relative to the file
    if (filePath !== undefined && rawOptions.projectPath !== undefined) {
        return paths_1.normalizeAndSlashify(path.join(path.dirname(filePath), rawOptions.projectPath));
    }
    // Otherwise give up and try a ./tsconfig.json relative to the package directory
    return paths_1.normalizeAndSlashify(path.join(cwd, "tsconfig.json"));
};
const collectFileNames = (argv, cwd, rawOptions) => __awaiter(void 0, void 0, void 0, function* () {
    if (argv.args !== undefined && argv.args.length !== 0) {
        return [argv.args, yield glob_1.globAllAsync(argv.args)];
    }
    if (rawOptions.include === undefined) {
        return undefined;
    }
    return [rawOptions.include, yield glob_1.globAllAsync(rawOptions.include.map((include) => path.join(cwd, include)))];
});
const implicitNodeModulesIncluded = (globs, fileNames) => {
    return !globs.some((glob) => glob.includes("node_modules")) && (fileNames === null || fileNames === void 0 ? void 0 : fileNames.find((fileName) => fileName.includes("node_modules")));
};
//# sourceMappingURL=loadOptions.js.map