"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectAddedMutators = void 0;
const path = __importStar(require("path"));
const builtIn_1 = require("../../mutators/builtIn");
const arrays_1 = require("../../shared/arrays");
const errors_1 = require("../../shared/errors");
/**
 * Finds mutators to use in runtime, as either the built-in mutators or custom mutators specified by the user.
 *
 * @param rawOptions   Options listed as JSON in a typestat configuration file.
 * @param packageDirectory   Base directory to resolve paths from.
 * @param output   Wraps process.stderr and process.stdout.
 * @returns Mutators to run with their friendly names.
 */
const collectAddedMutators = (rawOptions, packageDirectory, output) => {
    const addedMutators = arrays_1.collectOptionals(rawOptions.mutators);
    if (addedMutators.length === 0) {
        return builtIn_1.builtInFileMutators;
    }
    const additions = [];
    for (const rawAddedMutator of addedMutators) {
        try {
            const addedMutator = collectAddedMutator(packageDirectory, rawAddedMutator, output);
            if (addedMutator !== undefined) {
                additions.push([rawAddedMutator, addedMutator]);
            }
        }
        catch (error) {
            output.stderr(`Could not require ${rawAddedMutator} from ${packageDirectory}.`);
            output.stderr(errors_1.getQuickErrorSummary(error));
        }
    }
    return additions;
};
exports.collectAddedMutators = collectAddedMutators;
const collectAddedMutator = (packageDirectory, rawAddedMutator, output) => {
    const requiringPath = path.join(packageDirectory, rawAddedMutator);
    const resolvedImport = tryRequireResolve(requiringPath);
    if (resolvedImport === undefined) {
        output.stderr(`Could not require ${rawAddedMutator} at ${requiringPath} from ${packageDirectory}.`);
        output.stderr("It doesn't seem to exist? :(");
        return undefined;
    }
    const result = require(requiringPath);
    if (typeof result.fileMutator !== "function") {
        output.stderr(`Could not require ${rawAddedMutator} from ${packageDirectory}.`);
        if (result.fileMutator === undefined) {
            output.stderr(`It doesn't have an exported .fileMutator, which must be a function.`);
        }
        else {
            output.stderr(`Its exported .fileMutator is ${typeof result.fileMutator} intead of function.`);
        }
        return undefined;
    }
    return result.fileMutator;
};
const tryRequireResolve = (requiringPath) => {
    try {
        return require.resolve(requiringPath);
    }
    catch (error) {
        return undefined;
    }
};
//# sourceMappingURL=collectAddedMutators.js.map