"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processFileRenames = void 0;
const mz_1 = require("mz");
const processFileRenames = (options) => __awaiter(void 0, void 0, void 0, function* () {
    if (typeof options === "string" || options.fileNames === undefined) {
        return options;
    }
    return options.files.renameExtensions ? renameOptionFiles(options, options.fileNames) : ensureNoJsFiles(options, options.fileNames);
});
exports.processFileRenames = processFileRenames;
const renameOptionFiles = (options, fileNames) => __awaiter(void 0, void 0, void 0, function* () {
    const newFileNames = new Set(options.fileNames);
    const filesToRename = (yield Promise.all(fileNames.filter(canBeRenamed).map((fileName) => __awaiter(void 0, void 0, void 0, function* () {
        return ({
            newFileName: yield getNewFileName(options, fileName),
            oldFileName: fileName,
        });
    })))).filter(({ newFileName, oldFileName }) => newFileName !== oldFileName);
    if (filesToRename.length === 0) {
        return options;
    }
    options.output.stdout(`Renaming ${filesToRename.length} files...`);
    for (const { oldFileName, newFileName } of filesToRename) {
        yield mz_1.fs.rename(oldFileName, newFileName);
        newFileNames.delete(oldFileName);
        newFileNames.add(newFileName);
    }
    return Object.assign(Object.assign({}, options), { fileNames: Array.from(newFileNames) });
});
const validRenameExtensions = new Set([".js", ".jsx"]);
const canBeRenamed = (oldFileName) => {
    const oldExtension = oldFileName.substring(oldFileName.lastIndexOf("."));
    return validRenameExtensions.has(oldExtension);
};
const getNewFileName = (options, oldFileName) => __awaiter(void 0, void 0, void 0, function* () {
    const oldExtension = oldFileName.substring(oldFileName.lastIndexOf("."));
    const beforeExtension = oldFileName.substring(0, oldFileName.length - oldExtension.length);
    if (options.files.renameExtensions === "tsx" || oldExtension === ".jsx") {
        return `${beforeExtension}.tsx`;
    }
    if (options.files.renameExtensions === "ts") {
        return `${beforeExtension}.ts`;
    }
    const fileContents = (yield mz_1.fs.readFile(oldFileName)).toString();
    const fileContentsJoined = fileContents.replace(/ /g, "").replace(/"/g, "'");
    if (fileContentsJoined.includes(`require('react')`) || fileContentsJoined.includes(`from'react'`)) {
        return `${beforeExtension}.tsx`;
    }
    return `${beforeExtension}.ts`;
});
const ensureNoJsFiles = (options, fileNames) => __awaiter(void 0, void 0, void 0, function* () {
    const jsFileNames = fileNames.filter((fileName) => fileName.endsWith(".js") || fileName.endsWith(".jsx"));
    if (jsFileNames.length === 0) {
        return options;
    }
    return [
        "The following JavaScript files were included in the project but files.renameExtensions is not enabled.",
        "TypeStat does not yet support annotating JavaScript files.",
        ...jsFileNames.map((fileName) => `\t${fileName}`),
        "See https://github.com/JoshuaKGoldberg/TypeStat/blob/main/docs/Files.md for details.",
    ].join("\n");
});
//# sourceMappingURL=processFileRenames.js.map