"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createProcessOutput = void 0;
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const mz_1 = require("mz");
const os_1 = require("os");
const createProcessOutput = (logFile) => {
    const log = createLogfileOutput(logFile);
    const wrapStream = (prefix, stream) => {
        return (line) => {
            stream.write(line + os_1.EOL);
            log(strip_ansi_1.default(`[${prefix}] ${line}${os_1.EOL}`));
        };
    };
    return {
        log,
        stderr: wrapStream("stderr", process.stderr),
        stdout: wrapStream("stdout", process.stdout),
    };
};
exports.createProcessOutput = createProcessOutput;
const createLogfileOutput = (logFile) => {
    if (!logFile) {
        return () => { };
    }
    const file = mz_1.fs.openSync(logFile, "w");
    return (line) => {
        mz_1.fs.appendFileSync(file, line + os_1.EOL);
    };
};
//# sourceMappingURL=output.js.map