"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findMutationsInFile = void 0;
const chalk_1 = __importDefault(require("chalk"));
const os_1 = require("os");
const complaint_1 = require("../mutators/complaint");
const runtime_1 = require("../shared/runtime");
/**
 * Collects all mutations that should apply to a file.
 */
const findMutationsInFile = (request) => __awaiter(void 0, void 0, void 0, function* () {
    const checkMessage = chalk_1.default.grey(`Checking ${chalk_1.default.bold(request.sourceFile.fileName)}...`);
    request.options.output.stdout(checkMessage);
    let mutations = runtime_1.findFirstMutations(request, request.options.mutators);
    if (mutations instanceof complaint_1.MutationsComplaint) {
        request.options.output.stderr(`${os_1.EOL}Error in ${request.sourceFile.fileName} with ${mutations.mutatorPath.join(" > ")}: ${mutations.error.stack}${os_1.EOL}${os_1.EOL}`);
        mutations = undefined;
    }
    return mutations;
});
exports.findMutationsInFile = findMutationsInFile;
//# sourceMappingURL=findMutationsInFile.js.map