"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createInstallMissingTypesProvider = void 0;
const builtin_modules_1 = __importDefault(require("builtin-modules"));
const sets_1 = require("../../shared/sets");
const createFileNamesAndServices_1 = require("../createFileNamesAndServices");
const createSingleUserProvider_1 = require("../createSingleUserProvider");
const collectExistingTypingPackages_1 = require("./missingTypes/collectExistingTypingPackages");
const collectPackageManagerRunner_1 = require("./missingTypes/collectPackageManagerRunner");
const collectReferencedPackageNames_1 = require("./missingTypes/collectReferencedPackageNames");
const filterTypedPackageNames_1 = require("./missingTypes/filterTypedPackageNames");
const uniqueBuiltinModules = new Set(builtin_modules_1.default);
/**
 * Creates a mutations provider that installs missing types modules.
 *
 * @param options   Parsed runtime options for TypeStat.
 * @returns Provider to install missing types modules.
 */
const createInstallMissingTypesProvider = (options) => {
    const installMissingTypes = () => __awaiter(void 0, void 0, void 0, function* () {
        if (options.package.missingTypes === undefined) {
            return;
        }
        // Collect package names already present in the package file
        const existingPackageNames = yield collectExistingTypingPackages_1.collectExistingTypingPackages(options, options.package.file);
        if (existingPackageNames === undefined) {
            return;
        }
        // Collect every package name referenced by every file in the project
        const { services } = createFileNamesAndServices_1.createFileNamesAndServices(options);
        const referencedPackageNames = collectReferencedPackageNames_1.collectReferencedPackageNames(services);
        // Ignore package names already referenced in package.json or that don't exist in DefinitelyTyped
        const missingPackageNames = sets_1.setSubtract(referencedPackageNames, new Set(existingPackageNames), uniqueBuiltinModules);
        const missingTypedPackageNames = yield filterTypedPackageNames_1.filterTypedPackageNames(Array.from(missingPackageNames));
        if (missingTypedPackageNames.length === 0) {
            return;
        }
        // Run the installation command using the requested or detected package manager
        const packageManagerRunner = yield collectPackageManagerRunner_1.collectPackageManagerRunner(options, options.package.missingTypes);
        yield packageManagerRunner(options, missingTypedPackageNames);
    });
    return createSingleUserProvider_1.createSingleUseProvider(() => __awaiter(void 0, void 0, void 0, function* () {
        yield installMissingTypes();
        return {
            fileMutations: undefined,
        };
    }));
};
exports.createInstallMissingTypesProvider = createInstallMissingTypesProvider;
//# sourceMappingURL=createInstallMissingTypesProvider.js.map