"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMarkFilesModifiedProvider = void 0;
const fs = __importStar(require("mz/fs"));
const newlines_1 = require("../../shared/printing/newlines");
const createSingleUserProvider_1 = require("../createSingleUserProvider");
/**
 * Creates a mutations wave to mark all previously mutated files as modified.
 *
 * @param options   Parsed runtime options for TypeStat.
 * @param allModifiedFileNames   Unique names of all files that were modified.
 * @returns Mutations wave marking all mutated files as modified.
 */
const createMarkFilesModifiedProvider = (options, allModifiedFileNames) => {
    return createSingleUserProvider_1.createSingleUseProvider(() => __awaiter(void 0, void 0, void 0, function* () {
        if (options.files.above === "" && options.files.below === "") {
            return {
                fileMutations: undefined,
            };
        }
        const fileMutations = {};
        let hadMutation = false;
        for (const fileName of allModifiedFileNames) {
            const mutations = yield createFileMutations(options, fileName);
            if (mutations.length !== 0) {
                fileMutations[fileName] = mutations;
                hadMutation = true;
            }
        }
        return { fileMutations: hadMutation ? fileMutations : undefined };
    }));
};
exports.createMarkFilesModifiedProvider = createMarkFilesModifiedProvider;
const createFileMutations = (options, fileName) => __awaiter(void 0, void 0, void 0, function* () {
    const mutations = [];
    const fileContents = (yield fs.readFile(fileName)).toString();
    const fileContentsTrimmed = fileContents.trim();
    const newLine = newlines_1.printNewLine(options.compilerOptions);
    if (options.files.above !== "" && !fileContentsTrimmed.startsWith(options.files.above)) {
        mutations.push({
            insertion: `${options.files.above}${newLine}`,
            range: {
                begin: 0,
                end: 0,
            },
            type: "text-insert",
        });
    }
    if (options.files.below !== "" && !fileContentsTrimmed.endsWith(options.files.below)) {
        mutations.push({
            insertion: `${newLine}${options.files.below}`,
            range: {
                begin: getInsertionIndexBeforeLastEndline(fileContents),
            },
            type: "text-insert",
        });
    }
    return mutations;
});
const getInsertionIndexBeforeLastEndline = (fileContents) => {
    if (fileContents.length === 0) {
        return 0;
    }
    let index = fileContents.length - 1;
    for (const character of ["\n", "\r"]) {
        if (fileContents[index] === character) {
            index -= 1;
            if (index === 0) {
                return 0;
            }
        }
    }
    return index + 1;
};
//# sourceMappingURL=createMarkFilesModifiedProvider.js.map