"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRequireRenameProvider = void 0;
const findRequireRenameMutationsInFile_1 = require("../../mutations/renames/findRequireRenameMutationsInFile");
const maps_1 = require("../../shared/maps");
const createFileNamesAndServices_1 = require("../createFileNamesAndServices");
const createSingleUserProvider_1 = require("../createSingleUserProvider");
/**
 * Creates a mutations provider that transforms local require() calls in files.
 *
 * @param options   Parsed runtime options for TypeStat.
 * @param allModifiedFileNames   Set to mark names of all files that were modified.
 */
const createRequireRenameProvider = (options, allModifiedFiles) => {
    return createSingleUserProvider_1.createSingleUseProvider(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!options.files.renameExtensions) {
            return {
                fileMutations: undefined,
            };
        }
        const fileMutations = new Map();
        const { fileNames, services } = createFileNamesAndServices_1.createFileNamesAndServices(options);
        const allFileNames = new Set(fileNames);
        for (const fileName of fileNames) {
            const sourceFile = services.program.getSourceFile(fileName);
            if (sourceFile === undefined) {
                options.output.stderr(`Could not find TypeScript source file for '${fileName}'.`);
                continue;
            }
            const foundMutations = findRequireRenameMutationsInFile_1.findRequireRenameMutationsInFile({
                allFileNames,
                options,
                sourceFile,
            });
            if (foundMutations.length !== 0) {
                allModifiedFiles.add(fileName);
                fileMutations.set(fileName, foundMutations);
            }
        }
        return {
            fileMutations: fileMutations.size === 0 ? undefined : maps_1.convertMapToObject(fileMutations),
        };
    }));
};
exports.createRequireRenameProvider = createRequireRenameProvider;
//# sourceMappingURL=createRequireRenameProvider.js.map