"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectExistingTypingPackages = void 0;
const mz_1 = require("mz");
const os_1 = require("os");
const errors_1 = require("../../../shared/errors");
const collectExistingTypingPackages = (options, packagePath) => __awaiter(void 0, void 0, void 0, function* () {
    const allDependencies = yield tryCollectAllDependencies(packagePath);
    if (typeof allDependencies === "string") {
        options.output.stderr(`${os_1.EOL}Error trying to collect existing dependencies for --packageMissingTypes:`);
        options.output.stderr(`\t${allDependencies}`);
        options.output.stderr(os_1.EOL);
        return undefined;
    }
    return Array.from(allDependencies)
        .filter((dependency) => dependency.startsWith("@types/"))
        .map((dependency) => dependency.substring("@types/".length));
});
exports.collectExistingTypingPackages = collectExistingTypingPackages;
const tryCollectAllDependencies = (packagePath) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return collectAllDependencies(packagePath);
    }
    catch (error) {
        return errors_1.getQuickErrorSummary(error);
    }
});
const collectAllDependencies = (packagePath) => __awaiter(void 0, void 0, void 0, function* () {
    const rawContents = (yield mz_1.fs.readFile(packagePath)).toString();
    const parsedContents = JSON.parse(rawContents);
    const allDependencies = [];
    for (const groupName of ["dependencies", "devDependencies", "peerDependencies"]) {
        const packageObject = parsedContents[groupName];
        if (packageObject !== undefined) {
            allDependencies.push(...Object.keys(packageObject));
        }
    }
    return new Set(allDependencies);
});
//# sourceMappingURL=collectExistingTypingPackages.js.map