"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.collectReferencedPackageNames = void 0;
const ts = __importStar(require("typescript"));
const collectReferencedPackageNames = (services) => {
    const packageNames = new Set();
    for (const sourceFile of services.program.getSourceFiles()) {
        for (const packageName of collectFileReferencedPackageNames(sourceFile)) {
            packageNames.add(packageName);
        }
    }
    return packageNames;
};
exports.collectReferencedPackageNames = collectReferencedPackageNames;
const collectFileReferencedPackageNames = (sourceFile) => {
    const packageNames = new Set();
    const visitNode = (node) => {
        const packageName = parsePackageNameFromNode(node);
        if (packageName !== undefined) {
            packageNames.add(packageName);
        }
        ts.forEachChild(node, visitNode);
    };
    ts.forEachChild(sourceFile, visitNode);
    return packageNames;
};
const parsePackageNameFromNode = (node) => {
    if (ts.isIdentifier(node) && (node.text === "module" || node.text === "process")) {
        return "node";
    }
    if (ts.isImportDeclaration(node)) {
        return parseImportDeclarationPackageName(node);
    }
    if (ts.isImportEqualsDeclaration(node)) {
        return parseImportEqualsDeclarationPackageName(node);
    }
    if (ts.isCallExpression(node)) {
        return parseCallExpressionPackageName(node);
    }
    return undefined;
};
const parseImportDeclarationPackageName = (node) => parseModuleSpecifier(node.moduleSpecifier);
const parseImportEqualsDeclarationPackageName = (node) => {
    const { moduleReference } = node;
    if (!ts.isExternalModuleReference(moduleReference)) {
        return undefined;
    }
    return parseModuleSpecifier(moduleReference.expression);
};
const parseCallExpressionPackageName = (node) => {
    if (!ts.isIdentifier(node.expression) ||
        (node.expression.text !== "import" && node.expression.text !== "require") ||
        node.arguments.length !== 1) {
        return undefined;
    }
    const firstArgument = node.arguments[0];
    if (!ts.isStringLiteral(firstArgument)) {
        return undefined;
    }
    return parseModuleSpecifier(firstArgument);
};
const parseModuleSpecifier = (node) => {
    if (!ts.isStringLiteral(node)) {
        return undefined;
    }
    const { text } = node;
    if (text[0].match(/[aZ]/) === null) {
        return undefined;
    }
    return text.split("/")[0];
};
//# sourceMappingURL=collectReferencedPackageNames.js.map