"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLanguageServices = void 0;
const fs = __importStar(require("mz/fs"));
const ts = __importStar(require("typescript"));
const arrays_1 = require("../shared/arrays");
const typeNodes_1 = require("../shared/typeNodes");
const createProgramConfiguration_1 = require("./createProgramConfiguration");
/**
 * @returns Associated language service and type information based on TypeStat options.
 */
const createLanguageServices = (options) => {
    // Collect file names and parse raw options into a TypeScript program with its configuration settings
    const { fileNames, parsedConfiguration, program } = createProgramConfiguration_1.createProgramConfiguration(options);
    // Create a TypeScript language service using the compiler host
    const servicesHost = {
        fileExists: ts.sys.fileExists,
        getCompilationSettings: () => options.compilerOptions,
        getCurrentDirectory: () => options.package.directory,
        getDefaultLibFileName: ts.getDefaultLibFilePath,
        getScriptFileNames: () => fileNames,
        getScriptSnapshot: (fileName) => fs.existsSync(fileName) ? ts.ScriptSnapshot.fromString(fs.readFileSync(fileName).toString()) : undefined,
        getScriptVersion: () => "0",
        readDirectory: ts.sys.readDirectory,
        readFile: ts.sys.readFile,
    };
    const languageService = ts.createLanguageService(servicesHost, ts.createDocumentRegistry());
    // This printer will later come in handy for emitting raw ASTs to text
    const printer = ts.createPrinter({
        newLine: options.compilerOptions.newLine,
    });
    const printers = {
        node(node) {
            return printer.printNode(ts.EmitHint.Unspecified, node, ts.createSourceFile("temp.ts", "", ts.ScriptTarget.Latest, false, ts.ScriptKind.TSX));
        },
        type(types, enclosingDeclaration, typeFormatFlags) {
            const typeChecker = program.getTypeChecker();
            return arrays_1.uniquify(...arrays_1.arrayify(types).map((type) => typeof type === "string"
                ? type
                : typeChecker.typeToString(
                // Our mutations generally always go for base primitives, not literals
                // This might need to be revisited for potential future high fidelity types...
                typeChecker.getBaseTypeOfLiteralType(type), enclosingDeclaration, typeFormatFlags))).join(" | ");
        },
    };
    let wellKnownTypes;
    return {
        languageService,
        parsedConfiguration,
        program,
        printers,
        get wellKnownTypes() {
            return (wellKnownTypes !== null && wellKnownTypes !== void 0 ? wellKnownTypes : (wellKnownTypes = program.getTypeCatalog().reduce((acc, type) => {
                if (typeNodes_1.isIntrisinicNameType(type)) {
                    acc[type.intrinsicName] = type;
                }
                return acc;
            }, {})));
        },
    };
};
exports.createLanguageServices = createLanguageServices;
//# sourceMappingURL=language.js.map