"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getManuallyAssignedTypeOfNode = void 0;
const ts = __importStar(require("typescript"));
const functionTypes_1 = require("./functionTypes");
const types_1 = require("./types");
/**
 * Given a node passed to a location with a declared, known type,
 * attempts to find that known type.
 *
 * @example
 * If the node is assigned to a variable, this would return that variable's type.
 * In this case, that would be `Record<string, string>`:
 *
 * ```ts
 * const value: Record<string, string> = {};
 *                                    // ^^
 * ```
 *
 * @example
 * If the node passed to a function, this would return that variable's type.
 * In this case, that would be `number | null`:
 *
 * ```ts
 * function receivesNumber(value: number | null) { }
 * receivesNumber(3);
 *             // ^
 * ```
 */
const getManuallyAssignedTypeOfNode = (request, node) => {
    const { parent } = node;
    if (ts.isCallExpression(parent)) {
        return getAssignedTypeOfParameter(request, node, parent);
    }
    if (ts.isVariableDeclaration(parent)) {
        return getAssignedTypeOfVariable(request, parent);
    }
    return undefined;
};
exports.getManuallyAssignedTypeOfNode = getManuallyAssignedTypeOfNode;
const getAssignedTypeOfParameter = (request, node, parent) => {
    // Collect the declared type of the function-like being called
    const functionLikeValueDeclaration = functionTypes_1.getValueDeclarationOfFunction(request, parent.expression);
    if (functionLikeValueDeclaration === undefined) {
        return undefined;
    }
    // Don't bother checking for an out-of-range parameter
    const argumentIndex = parent.arguments.indexOf(node);
    if (argumentIndex >= functionLikeValueDeclaration.parameters.length) {
        return undefined;
    }
    return types_1.getTypeAtLocationIfNotError(request, functionLikeValueDeclaration.parameters[argumentIndex]);
};
const getAssignedTypeOfVariable = (request, node) => {
    if (node.type === undefined) {
        return undefined;
    }
    return request.services.program.getTypeChecker().getTypeFromTypeNode(node.type);
};
//# sourceMappingURL=assignments.js.map