"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.declaredInitializedTypeNodeIsRedundant = void 0;
const tsutils_1 = require("tsutils");
const ts = __importStar(require("typescript"));
const typeNodes_1 = require("./typeNodes");
const types_1 = require("./types");
const declaredInitializedTypeNodeIsRedundant = (request, declaration, initializer) => {
    // Most literals (e.g. `""`) have a corresponding keyword (e.g. `string`)
    switch (declaration.kind) {
        case ts.SyntaxKind.BooleanKeyword:
            return initializer.kind === ts.SyntaxKind.FalseKeyword || initializer.kind === ts.SyntaxKind.TrueKeyword;
        case ts.SyntaxKind.NullKeyword:
            return initializer.kind === ts.SyntaxKind.NullKeyword;
        case ts.SyntaxKind.NumberKeyword:
            return initializer.kind === ts.SyntaxKind.NumericLiteral;
        case ts.SyntaxKind.StringKeyword:
            return initializer.kind === ts.SyntaxKind.StringLiteral;
        // (except for `undefined`, which is an initializer one should never reassign)
        case ts.SyntaxKind.UndefinedKeyword:
            return ts.isIdentifier(initializer) && initializer.text === "undefined";
    }
    // `RegExp`s are also initializers that one should never reassign
    if (ts.isTypeReferenceNode(declaration) && ts.isIdentifier(declaration.typeName)) {
        switch (declaration.typeName.text) {
            case "RegExp":
                return initializer.kind === ts.SyntaxKind.RegularExpressionLiteral;
        }
    }
    // Other types are complex enough to need the type checker...
    const declaredType = types_1.getTypeAtLocationIfNotError(request, declaration);
    if (declaredType === undefined) {
        return undefined;
    }
    const initializedType = types_1.getTypeAtLocationIfNotError(request, initializer);
    if (initializedType === undefined) {
        return undefined;
    }
    return declaredTypeIsEquivalent(request.services.program.getTypeChecker(), declaredType, initializedType);
};
exports.declaredInitializedTypeNodeIsRedundant = declaredInitializedTypeNodeIsRedundant;
const declaredTypeIsEquivalent = (typeChecker, declaredType, initializedType) => {
    // Most types, such as `string[]` / `[""]`, are generally found by this intersection...
    if (typeChecker.isTypeAssignableTo(declaredType, initializedType) &&
        typeChecker.isTypeAssignableTo(initializedType, declaredType) &&
        // ...though, notably, declares union types trigger false positives against non-union initializations
        !(tsutils_1.isUnionType(declaredType) && !tsutils_1.isUnionType(initializedType))) {
        return true;
    }
    // We have to hackily check for boolean types with intrinsic names...
    // `boolean[]` is really the type `[false, true]`
    if (typeNodes_1.isIntrisinicNameType(declaredType) && typeNodes_1.isIntrisinicNameType(initializedType)) {
        return intrinsicNamesAreEquivalent(declaredType.intrinsicName, initializedType.intrinsicName);
    }
    // If the types have the same symbol, they're likely arrays or classes with potentially comparable generics
    return typeSymbolsAndArgumentsAreEquivalent(typeChecker, declaredType, initializedType);
};
const typeSymbolsAndArgumentsAreEquivalent = (typeChecker, declaredType, initializedType) => {
    const declaredSymbol = declaredType.getSymbol();
    const initializedSymbol = initializedType.getSymbol();
    if (declaredSymbol !== initializedSymbol ||
        !typeNodes_1.isOptionalTypeArgumentsTypeNode(declaredType) ||
        declaredType.typeArguments === undefined ||
        !typeNodes_1.isOptionalTypeArgumentsTypeNode(initializedType) ||
        initializedType.typeArguments === undefined) {
        return false;
    }
    if (declaredType.typeArguments.length !== initializedType.typeArguments.length) {
        return false;
    }
    for (let i = 0; i < declaredType.typeArguments.length; i += 1) {
        const declaredTypeArgument = declaredType.typeArguments[i];
        const initializedTypeArgument = initializedType.typeArguments[i];
        if (!declaredTypeIsEquivalent(typeChecker, declaredTypeArgument, initializedTypeArgument)) {
            return false;
        }
    }
    return true;
};
/**
 * Checks intrinsic names of types that wouldn't have been caught with `typeChecker.isTypeAssignableTo`.
 */
const intrinsicNamesAreEquivalent = (declaredName, initializedName) => {
    switch (declaredName) {
        case "boolean":
            return initializedName === "false" || initializedName === "true";
        case "null":
            return initializedName === "null";
    }
    return false;
};
//# sourceMappingURL=comparisons.js.map