"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isNeverAndOrUnknownType = exports.isKnownGlobalBaseType = exports.getIdentifyingTypeLiteralParent = exports.isNullOrUndefinedMissingBetween = exports.isTypeMissingBetween = exports.getValueDeclarationOfType = exports.isPropertySignatureWithStaticName = exports.isNodeWithDefinedTypeParameters = exports.isNodeWithDefinedTypeArguments = exports.isNodeWithIdentifierName = exports.isNodeWithType = void 0;
const ts = __importStar(require("typescript"));
const flags_1 = require("../mutations/collecting/flags");
const types_1 = require("./types");
const isNodeWithType = (node) => node.type !== undefined;
exports.isNodeWithType = isNodeWithType;
const isNodeWithIdentifierName = (node) => {
    return "name" in node && !!node.name;
};
exports.isNodeWithIdentifierName = isNodeWithIdentifierName;
const isNodeWithDefinedTypeArguments = (node) => {
    return "typeArguments" in node;
};
exports.isNodeWithDefinedTypeArguments = isNodeWithDefinedTypeArguments;
const isNodeWithDefinedTypeParameters = (node) => {
    return "typeParameters" in node;
};
exports.isNodeWithDefinedTypeParameters = isNodeWithDefinedTypeParameters;
const isPropertySignatureWithStaticName = (node) => {
    return ts.isPropertySignature(node) && exports.isNodeWithIdentifierName(node);
};
exports.isPropertySignatureWithStaticName = isPropertySignatureWithStaticName;
const getValueDeclarationOfType = (request, node) => {
    var _a, _b;
    // Try getting the symbol at the location, which sometimes only works in the latter form
    const nodeType = types_1.getTypeAtLocationIfNotError(request, node);
    const symbol = (_a = nodeType === null || nodeType === void 0 ? void 0 : nodeType.getSymbol()) !== null && _a !== void 0 ? _a : request.services.program.getTypeChecker().getSymbolAtLocation(node);
    if (symbol === undefined) {
        return undefined;
    }
    // Despite the type definition, valueDeclaration itself is sometimes undefined
    const valueDeclaration = symbol.valueDeclaration;
    if (valueDeclaration !== undefined) {
        return valueDeclaration;
    }
    // Since the real .valueDeclaration is undefined, use the first one we know of
    return ((_b = symbol.declarations) === null || _b === void 0 ? void 0 : _b.length) ? symbol.declarations[0] : undefined;
};
exports.getValueDeclarationOfType = getValueDeclarationOfType;
/**
 * @returns Whether a type is missing in a passed type compared to its declared type.
 */
const isTypeMissingBetween = (typeFlag, passedType, declaredType) => flags_1.isTypeFlagSetRecursively(passedType, typeFlag) && !flags_1.isTypeFlagSetRecursively(declaredType, typeFlag);
exports.isTypeMissingBetween = isTypeMissingBetween;
// If either null or undefined is missing in the argument, we'll need a ! mutation
/**
 * @returns Whether null or undefined is missing in a passed type compared to its declared type.
 */
const isNullOrUndefinedMissingBetween = (passedType, declaredType) => exports.isTypeMissingBetween(ts.TypeFlags.Null, passedType, declaredType) ||
    exports.isTypeMissingBetween(ts.TypeFlags.Undefined, passedType, declaredType);
exports.isNullOrUndefinedMissingBetween = isNullOrUndefinedMissingBetween;
const getIdentifyingTypeLiteralParent = (node) => {
    const { parent } = node;
    if (ts.isTypeAliasDeclaration(parent)) {
        return parent.name;
    }
    // ???
    return node;
};
exports.getIdentifyingTypeLiteralParent = getIdentifyingTypeLiteralParent;
// Todo: eventually, these should expand to object, Object, etc...
// Those are stored as intrinsicNames on types, and were showing up without missing flags for some reason
const knownGlobalBaseTypeNames = new Set(["Function"]);
/**
 * @returns Whether the type is a known base type such as Function.
 */
const isKnownGlobalBaseType = (type) => {
    var _a;
    return knownGlobalBaseTypeNames.has((_a = type.getSymbol()) === null || _a === void 0 ? void 0 : _a.escapedName.toString());
};
exports.isKnownGlobalBaseType = isKnownGlobalBaseType;
const neverAndOrUnknown = [
    ts.TypeFlags.Never,
    ts.TypeFlags.Unknown,
    ts.TypeFlags.Never & ts.TypeFlags.Unknown,
    ts.TypeFlags.Never | ts.TypeFlags.Unknown,
];
const isNeverAndOrUnknownType = (type) => {
    return neverAndOrUnknown.includes(type.flags);
};
exports.isNeverAndOrUnknownType = isNeverAndOrUnknownType;
//# sourceMappingURL=nodeTypes.js.map