"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getClassExtendsType = exports.getCloseAncestorCallOrNewExpression = exports.getExpressionWithin = exports.getVariableInitializerForExpression = exports.isNodeAssigningBinaryExpression = exports.getParentOfKind = exports.isNodeWithinNode = exports.findNodeByStartingPosition = void 0;
const tsutils = __importStar(require("tsutils"));
const ts = __importStar(require("typescript"));
const nodeTypes_1 = require("./nodeTypes");
/**
 * Finds a node in a source file by its starting position.
 */
const findNodeByStartingPosition = (sourceFile, start) => {
    if (start >= sourceFile.end) {
        throw new Error(`Cannot request start ${start} outside of source file '${sourceFile.fileName}'.`);
    }
    const visitNode = (node) => {
        const nodeStart = node.getStart(sourceFile);
        if (nodeStart === start) {
            return node;
        }
        if (nodeStart > start || node.end < start) {
            return undefined;
        }
        return ts.forEachChild(node, visitNode);
    };
    return ts.forEachChild(sourceFile, visitNode);
};
exports.findNodeByStartingPosition = findNodeByStartingPosition;
/**
 * Checks whether a node's position is completely within a parent node's.
 */
const isNodeWithinNode = (sourceFile, child, parent) => child.end <= parent.end && child.getStart(sourceFile) >= parent.getStart(sourceFile);
exports.isNodeWithinNode = isNodeWithinNode;
const getParentOfKind = (node, selector) => {
    if (node.parent === undefined) {
        return undefined;
    }
    node = node.parent;
    if (selector(node)) {
        return node;
    }
    return exports.getParentOfKind(node.parent, selector);
};
exports.getParentOfKind = getParentOfKind;
/**
 * @returns Whether a node is a binary expression that sets a value with an equals token.
 * @remarks This only looks at = assignments, ignoring others such as +=.
 */
const isNodeAssigningBinaryExpression = (node) => ts.isBinaryExpression(node) && node.operatorToken.kind === ts.SyntaxKind.EqualsToken;
exports.isNodeAssigningBinaryExpression = isNodeAssigningBinaryExpression;
/**
 * Gets a variable initializer for an expression, if one exists.
 * If a parent is provided, it will ignore an initializer not within that parent.
 */
const getVariableInitializerForExpression = (request, expression, parentFunctionLike) => {
    if (!ts.isIdentifier(expression)) {
        return undefined;
    }
    const valueDeclaration = nodeTypes_1.getValueDeclarationOfType(request, expression);
    if (valueDeclaration === undefined ||
        (parentFunctionLike !== undefined && !exports.isNodeWithinNode(request.sourceFile, valueDeclaration, parentFunctionLike))) {
        return undefined;
    }
    if (!ts.isVariableDeclaration(valueDeclaration) || valueDeclaration.initializer === undefined) {
        return undefined;
    }
    return valueDeclaration.initializer;
};
exports.getVariableInitializerForExpression = getVariableInitializerForExpression;
const getExpressionWithin = (node) => (ts.isExpressionStatement(node) ? node.expression : node);
exports.getExpressionWithin = getExpressionWithin;
const getCloseAncestorCallOrNewExpression = (node) => {
    if (ts.isSourceFile(node.parent) || tsutils.isBlockLike(node)) {
        return undefined;
    }
    if (ts.isCallExpression(node) || ts.isNewExpression(node)) {
        return node;
    }
    return exports.getCloseAncestorCallOrNewExpression(node.parent);
};
exports.getCloseAncestorCallOrNewExpression = getCloseAncestorCallOrNewExpression;
const getClassExtendsType = (node) => {
    const { heritageClauses } = node;
    if (heritageClauses === undefined) {
        return undefined;
    }
    const classExtension = heritageClauses.find((clause) => clause.token === ts.SyntaxKind.ExtendsKeyword);
    if (classExtension === undefined) {
        return undefined;
    }
    return classExtension.types.length === 1 ? classExtension.types[0] : undefined;
};
exports.getClassExtendsType = getClassExtendsType;
//# sourceMappingURL=nodes.js.map