"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findRelevantNodeReferencesAsNodes = exports.isNodeFilteredOut = void 0;
const nodes_1 = require("./nodes");
const isNodeFilteredOut = (filteredNodes, node) => {
    while (node !== undefined) {
        if (filteredNodes.has(node)) {
            return true;
        }
        node = node.parent;
    }
    return false;
};
exports.isNodeFilteredOut = isNodeFilteredOut;
const findRelevantNodeReferencesAsNodes = (filteredNodes, services, identifyingNode) => {
    const references = findRelevantNodeReferences(filteredNodes, services, identifyingNode);
    if (references === undefined) {
        return undefined;
    }
    const referencingNodes = [];
    for (const reference of references) {
        // Grab the source file containing the reference
        const referencingSourceFile = services.program.getSourceFile(reference.fileName);
        if (referencingSourceFile === undefined) {
            continue;
        }
        // Find the referencing node from its place in that source file, unless it's the original node
        const referencingNode = nodes_1.findNodeByStartingPosition(referencingSourceFile, reference.textSpan.start);
        if (referencingNode === undefined || referencingNode === identifyingNode) {
            continue;
        }
        referencingNodes.push(referencingNode);
    }
    return referencingNodes;
};
exports.findRelevantNodeReferencesAsNodes = findRelevantNodeReferencesAsNodes;
const referenceIsFilteredOut = (filteredNodes, sourceFile, reference) => {
    return exports.isNodeFilteredOut(filteredNodes, nodes_1.findNodeByStartingPosition(sourceFile, reference.textSpan.start));
};
const findRelevantNodeReferences = (filteredNodes, services, node) => {
    // Find all locations the node is referenced
    const nodeSourceFile = node.getSourceFile();
    const referencedSymbols = services.languageService.findReferences(nodeSourceFile.fileName, node.getStart(nodeSourceFile));
    if (referencedSymbols === undefined) {
        return undefined;
    }
    const references = new Set();
    // For each reference within the referencing symbols, add it if it's not the child of a filtered node
    for (const referenceSymbol of referencedSymbols) {
        for (const reference of referenceSymbol.references) {
            // Add the reference if it's not a child of nodes we filter out
            const referencingSourceFile = services.program.getSourceFile(reference.fileName);
            if (referencingSourceFile !== undefined && !referenceIsFilteredOut(filteredNodes, referencingSourceFile, reference)) {
                references.add(reference);
            }
        }
    }
    return Array.from(references);
};
//# sourceMappingURL=references.js.map