"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTypeAtLocationIfNotError = exports.getSymbolAtLocationIfNotError = exports.isTypeBuiltIn = exports.typeHasLocalTypeParameters = void 0;
const typeNodes_1 = require("./typeNodes");
/**
 * @returns Whether the type has `localTypeParameters`, such as the built-in Map and Array definitions.
 */
const typeHasLocalTypeParameters = (type) => type.localTypeParameters !== undefined;
exports.typeHasLocalTypeParameters = typeHasLocalTypeParameters;
/**
 * @returns Whether a type is from the built-in .d.ts files shipped with TypeScript.
 */
const isTypeBuiltIn = (type) => {
    const symbol = type.getSymbol();
    if (!(symbol === null || symbol === void 0 ? void 0 : symbol.valueDeclaration)) {
        return false;
    }
    const sourceFile = symbol.valueDeclaration.getSourceFile();
    return sourceFile.hasNoDefaultLib && sourceFile.isDeclarationFile && sourceFile.fileName.includes("node_modules/typescript/lib/");
};
exports.isTypeBuiltIn = isTypeBuiltIn;
const getSymbolAtLocationIfNotError = (request, node) => {
    var _a;
    if (node === undefined) {
        return undefined;
    }
    const symbol = request.services.program.getTypeChecker().getSymbolAtLocation(node);
    return ((_a = symbol === null || symbol === void 0 ? void 0 : symbol.declarations) === null || _a === void 0 ? void 0 : _a.length) ? symbol : undefined;
};
exports.getSymbolAtLocationIfNotError = getSymbolAtLocationIfNotError;
const getTypeAtLocationIfNotError = (request, node) => {
    if (node === undefined) {
        return undefined;
    }
    const type = request.services.program.getTypeChecker().getTypeAtLocation(node);
    return typeNodes_1.isIntrisinicNameType(type) && type.intrinsicName === "error" ? undefined : type;
};
exports.getTypeAtLocationIfNotError = getTypeAtLocationIfNotError;
//# sourceMappingURL=types.js.map